/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.observer.Observable
	 *
	 * @classDescription
	 * The base <code>Observable</code> class.
	 *
	 * <P>
	 * An <code>Observable</code> is an object that will broadcast information to interested objects. It offers public
	 * methods for interested objects to be able to subscribe to broadcasting and others to allow classes to compose
	 * with.
	 * 
	 * @constructor
	 */
	Objs
	(
		"truemvc.observer.Observable",
		{
			/**
			 * @private
			 *
			 * List of <code>Observer</code> instances interested in <code>Notification</code>s.
			 *
			 * @type {Object}
			 */
			observerMap: null,

			/**
			 * Initialize an <code>Observable</code> instance.
			 */
			initialize: function()
			{
				this.observerMap = {};
			},

			/**
			 * Register an <code>Observer</code> to be notified of <code>Notification</code>s with a given name.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code>s to notify this <code>Observer</code> of.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> to register.
			 */
			registerObserver: function( name, observer )
			{
				var observers/*Array*/ = this.observerMap[name];
				if( observers )
					observers.push(observer);
				else
					this.observerMap[name] = [observer];
			},

			/**
			 * Notify the <code>Observer</code>s for a particular <code>Notification</code>.
			 *
			 * <P>
			 * All previously attached <code>Observer</code>s for this <code>Notification</code>'s list are notified and
			 * are passed a reference to the <code>Notification</code> in the order in which they were registered.
			 *
			 * @param {Notification} note
			 * 		The <code>Notification</code> to notify <code>Observer</code>s of.
			 */
			notifyObservers: function( note )
			{
				var name/*String*/ = note.getName();

				var observersRef/*Array*/ = this.observerMap[name];
				if( observersRef )
				{
					// Copy the array to avoid modification of it during the loop.
					var observers/*Array*/ = observersRef.slice(0);
					var len/*Number*/ = observers.length;
					for( var i/*Number*/=0; i<len; i++ )
					{
						var observer/*Observer*/ = observers[i];
						observer.notifyObserver(note);
					}
				}
			},

			/**
			 * Remove the <code>Observer</code> for a given <i>notifyContext</i> from an <code>Observer</code> list for
			 * a given <code>Notification</code> name.
			 *
			 * @param {String} name
			 * 		Which <code>Observer</code> list to remove from.
			 *
			 * @param {Object} notifyContext
			 * 		Remove the <code>Observer</code> with this object as its <i>notifyContext</i>.
			 */
			removeObserver: function( name, notifyContext )
			{
				var observers/*Array*/ = this.observerMap[name];
				var i/*Number*/ = observers.length;

				while( i-- )
				{
					var observer/*Observer*/ = observers[i];
					if( observer.compareNotifyContext(notifyContext) )
					{
						observers.splice( i, 1 );
						break;
					}
				}

				// Remove empty observer lists.
				if( observers.length == 0 )
					delete this.observerMap[name];
			}

			//TODO Remove all oberservers and add it to implementors
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.observer.Observer
	 *
	 * @classDescription
	 * The base <code>Observer</code> class.
	 *
	 * <P>
	 * An <code>Observer</code> is an object that encapsulates information about an interested object with a method that
	 * should be called when a particular <code>Notification</code> is broadcast.
	 *
	 * <P>
	 * In TrueMVC, the <code>Observer</code> class assumes these responsibilities:
	 *
	 * <UL>
	 * 		<LI>Encapsulate the notification (callback) method of the interested object.
	 * 		<LI>Encapsulate the notification context (this) of the interested object.
	 * 		<LI>Provide methods for setting the notification method and context.
	 * 		<LI>Provide a method for notifying the interested object.
	 *
	 * @see truemvc.mvc.View
	 * @see truemvc.observer.Notification
	 * 
	 * @constructor
	 */
	Objs
	(
		"truemvc.observer.Observer",
		{
			/**
			 * The notification method of the interested object.
			 * 
			 * @type {Function}
			 * @private
			 */
			notify: null,
			
			/**
			 * The notification context of the interested object.
			 * 
			 * @type {Object}
			 * @private
			 */
			context: null,
			
			/**
			 * Initialize an <code>Observer</code> instance.
			 *
			 * @param {Function} notifyMethod
			 * 		The notification method of the interested object.
			 * 
			 * @param {Object} notifyContext
			 * 		The notification context of the interested object.
			 */
			initialize: function( notifyMethod, notifyContext )
			{
				this.setNotifyMethod( notifyMethod );
				this.setNotifyContext( notifyContext );
			},
			
			/**
			 * Get the notification method.
			 *
			 * @return {Function}
			 * 		The notification (callback) method of the interested object.
			 */
			getNotifyMethod: function()
			{
				return this.notify;
			},
			
			/**
			 * Set the notification method.
			 *
			 * <P>The notification method should take one parameter of type <code>Notification</code>.
			 *
			 * @param {Function} notifyMethod
			 * 		The notification (callback) method of the interested object.
			 */
			setNotifyMethod: function( notifyMethod )
			{
				this.notify = notifyMethod;
			},
			
			/**
			 * Get the notification context.
			 *
			 * @return {Object}
			 * 		The notification context (<code>this</code>) of the interested object.
			 */
			getNotifyContext: function()
			{
				return this.context;
			},
			
			/**
			 * Set the notification context.
			 *
			 * @param {Object} notifyContext
			 * 		The notification context (this) of the interested object.
			 */
			setNotifyContext: function( notifyContext )
			{
				this.context = notifyContext;
			},
			
			/**
			 * Notify the interested object.
			 *
			 * @param {Notification} note
			 * 		The <code>Notification</code> to pass to the interested object's
			 * 		notification method.
			 */
			notifyObserver: function( note )
			{
				//FIXME Remove the note type and Notification dependency for this class and make the method accept an infinity of arguments
				this.getNotifyMethod().call( this.getNotifyContext(), note );
			},
			
			/**
			 * Compare an object to the notification context.
			 *
			 * @param {Object} object
			 * 		The object to compare.
			 *
			 * @return {Boolean}
			 * 		The object and the notification context are the same.
			 */
			compareNotifyContext: function( object )
			{
				return object === this.getNotifyContext();
			}
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.observer.Notification
	 *
	 * @classDescription
	 * The base <code>Notification</code> class.
	 *
	 * <P>
	 * TrueMVC does not rely upon underlying event models such as the one provided with Flash, and ActionScript 3 does
	 * not have an inherent event model.
	 *
	 * <P>
	 * The Observer pattern as implemented within TrueMVC exists to support event-driven communication between the
	 * application and the actors of the MVC triad (Model, View and Controller.
	 *
	 * <P>
	 * Notifications are not meant to be a replacement for Events in Flex/Flash/Air/Javascript. Generally,
	 * <code>Mediator</code> implementors place event listeners on their view components, which they then handle in the
	 * usual way. This may lead to the broadcast of <code>Notification</code>s to trigger <code>Command</code>s or to
	 * communicate with other <code>Mediators</code>. <code>Proxy</code> and <code>Command</code> instances communicate
	 * with each other and <code>Mediator</code>s by broadcasting <code>Notification</code>s.
	 *
	 * <P>
	 * A key difference between Flash <code>Event</code>s and TrueMVC <code>Notification</code>s is that
	 * <code>Event</code>s follow the 'Chain of Responsibility' pattern, 'bubbling' up the display hierarchy until some
	 * parent component handles the <code>Event</code>, while TrueMVC <code>Notification</code>s follow a
	 * 'Publish/Subscribe' pattern. TrueMVC classes need not be related to each other in a parent/child relationship in
	 * order to communicate with one another using <code>Notification</code>s.
	 *
	 * @see truemvc.observer.Observer
	 * 
	 * @constructor
	 */
	Objs
	(
		"truemvc.observer.Notification",
		{
			/**
			 * The name of the notification.
			 * 
			 * @type {String}
			 * @private 
			 */
			name: null,
			
			/**
			 * The body data to send with the notification.
			 * 
			 * @type {Object}
			 * @private
			 */
			body: null,
			
			/**
			 * The type identifier of the notification.
			 * 
			 * @type {String}
			 * @private
			 */
			type: null,
			
			/**
			 * Initialize a <code>Notification</code> instance.
			 *
			 * @param {String} name
			 * 		The name of the notification.
			 *
			 * @param {Object} body
			 * 		(optional) Body data to send with the notification.
			 * 
			 * @param {String} type (optional)
			 * 		Type identifier of the notification.
			 */
			initialize: function( name, body, type )
			{			
				this.name = name;
				this.body = body;
				this.type = type;
			},
			
			/**
			 * Get the name of the <code>Notification</code> instance.
			 *
			 * @return {String}
			 * 		The name of the <code>Notification</code> instance.
			 */
			getName: function()
			{
				return this.name;
			},
			
			/**
			 * Set the body of the <code>Notification</code> instance.
			 *
			 * @param {Object} body
			 * 		The body of the notification.
			 */
			setBody: function( body )
			{
				this.body = body;
			},
			
			/**
			 * Get the body of the <code>Notification</code> instance.
			 *
			 * @return {Object}
			 * 		The body for the notification.
			 */
			getBody: function()
			{
				return this.body;
			},
			
			/**
			 * Set the type of the <code>Notification</code> instance.
			 *
			 * @param {String} type
			 * 		The type identifier for the notification.
			 */
			setType: function( type )
			{
				this.type = type;
			},
			
			/**
			 * Get the type of the <code>Notification</code> instance.
			 *
			 * @return {String}
			 * 		The type identifier for the notification.
			 */
			getType: function()
			{
				return this.type;
			},
			
			/**
			 * Get a textual representation of the <code>Notification</code> instance.
			 *
			 * @return {String}
			 * 		The textual representation of the <code>Notification</code>	instance.
			 */
			toString: function()
			{
				var msg/*String*/ = "Notification Name: " + this.getName();
				msg += "\nBody:" + (( this.getBody() == null ) ? "null" : this.getBody().toString());
				msg += "\nType:" + (( this.getType() == null ) ? "null" : this.getType());
			
				return msg;
			}
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.observer.Notifier
	 * @abstract
	 *
	 * @classDescription
	 * The Base <code>Notifier</code> class.
	 *
	 * <P>
	 * <code>MacroCommand</code>, <code>Command</code>, <code>Mediator</code> and <code>Proxy</code> all have a need to
	 * send <code>Notifications</code>.
	 * 
	 * <P>
	 * The <code>Notifier</code> base class provides a common method called <code>sendNotification</code> that relieves
	 * implementation code of the necessity to actually construct <code>Notification</code>s.
	 *
	 * <P>
	 * The <code>Notifier</code> class, which all of the above mentioned classes extend, provides an initialized
	 * reference to the <code>Facade</code>, which is required by the convenience method <code>sendNotification</code>
	 * for sending <code>Notifications</code>, but it also eases implementation as these classes have frequent
	 * <code>Facade</code> interactions and uusually require access to the facade anyway.
	 * 
	 * @see truemvc.facade.Facade Facade
	 * 
	 * @constructor
	 */
	var Notifier = Objs
	(
		"truemvc.observer.Notifier",
		{

			/**
			 * @abstract
			 * Create and send a <code>Notification</code>.
			 *
			 * <P>
			 * Keeps us from having to construct new <code>Notification</code>
			 * instances in our implementation code.
			 * 
			 * @param {String} name
			 * 		The name of the notification to send.
			 * 
			 * @param {Object} body
			 * 		The (optional) body of the notification.
			 *
			 * @param {String} type
			 * 		The (optional) type of the notification.
			 */
			sendNotification: function( name, body, type )
			{
				throw Error(Notifier.SEND_NOTIFICATION_NOT_IMPLEMENTED_ERROR);
			}
		}
	);

	/**
	 * Error thrown when a subclass try to call sendNotification method without having implemented it.
	 *
	 * @type {String}
	 */
	Notifier.SEND_NOTIFICATION_NOT_IMPLEMENTED_ERROR = "Notifier.sendNotification method is abstract and must be overridden.";
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Notification = Objs("truemvc.observer.Notification");

	/**
	 * @class truemvc.command.Command
	 *
	 * @classDescription
	 * A base <code>Command</code> implementation.
	 * 
	 * <P>
	 * Your subclass should override the <code>execute</code> method where your business logic will handle the
	 * <code>Notification</code>.
	 *
	 * <P>
	 * As in JavaScript there isn't interfaces, <code>Command</code> and <code>MacroCommand</code> cannot offer
	 * the guarantee to have the right signature. We could have inherited from a common <code>Command</code> class,
	 * but to avoid an unwanted complexity and to respect TrueMVC implementation, this is to the developer to take care
	 * to inherit from <code>Command</code> in its command and <code>MacroCommand</code> depending on his need.
	 * 
	 * @extends truemvc.observer.Notifier Notifier
	 *
	 * @constructor
	 */
	var Command = Objs
	(
		"truemvc.command.Command",
		"truemvc.observer.Notifier",
		{

			/**
			 * The <code>View</code> this <code>Command</code> has to know.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,

			/**
			 * The <code>Model</code> this <code>Command</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * The <code>Controller</code> this <code>Command</code> has to know.
			 *
			 * @type {Controller}
			 * @private
			 */
			controller: null,

			/**
			 * Get the <code>Model</code> instance for this <code>Command</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Command</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Command</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Command</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * Get the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Command</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Command</code>.
			 */
			setView: function( view )
			{
				this.view = view;
			},

			/**
			 * Get the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @return {Controller}
			 * 		The <code>Controller</code> instance set for this <code>Command</code>.
			 */
			getController: function()
			{
				return this.controller;
			},

			/**
			 * Set the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @param {Controller} controller
			 * 		The <code>Model</code> instance to set for this <code>Command</code>.
			 */
			setController: function( controller )
			{
				this.controller = controller;
			},

			/**
			 * @override
			 */
			sendNotification: function( name, body, type )
			{
				var note/*Notification*/ = new Notification( name, body, type );
				this.view.notifyObservers(note);
			},

			/**
			 * @abstract
			 *
			 * Fulfill the use-case initiated by the given <code>Notification</code>.
			 *
			 * <P>
			 * In the Command Pattern, an application use-case typically begins with some user action, which results in
			 * a <code>Notification</code> being broadcast, which is handled by business logic in the
			 * <code>execute</code> method of an <code>Command</code>.
			 *
			 * @param {Notification} note 
			 * 		The <code>Notification</code> to handle.
			 */
			execute: function( note )
			{
				throw Error("Command.execute method is abstract and must be overridden.");
			}
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Notification = Objs("truemvc.observer.Notification");

	/**
	 * @class truemvc.command.MacroCommand
	 *
	 * @classDescription
	 * A base <code>Command</code> implementation that executes other <code>Command</code>s.
	 *
	 * <P>
	 * A <code>MacroCommand</code> maintains an list of <code>Command</code> class references called <i>SubCommands</i>.
	 *
	 * <P>
	 * When <code>execute</code> is called, the <code>MacroCommand</code> instantiates and calls <code>execute</code> on
	 * each of its <i>SubCommands</i> turn. Each <i>SubCommand</i> will be passed a reference to the original
	 * <code>Notification</code> that was passed to the <code>MacroCommand</code>'s <code>execute</code> method.
	 *
	 * <P>
	 * Unlike <code>MacroCommand</code>, your subclass should not override <code>execute</code>, but instead, should
	 * override the <code>initializeMacroCommand</code> method, calling <code>addSubCommand</code> once for each
	 * <i>SubCommand</i> to be executed.
	 *
	 * <P>
	 * As in JavaScript there isn't interfaces, <code>MacroCommand</code> and <code>MacroCommand</code> cannot offer
	 * the guarantee to have the right signature. We could have inherited from a common <code>Command</code> class,
	 * but to avoid an unwanted complexity and to respect TrueMVC implementation, this is to the developer to take care
	 * to inherit from <code>MacroCommand</code> in its command and <code>MacroCommand</code> depending on his need.
	 *
	 * @extends truemvc.observer.Notifier Notifier
	 *
	 * @constructor
	 */
	var MacroCommand = Objs
	(
		"truemvc.command.MacroCommand",
		"truemvc.command.Command",
		{
			/**
			 * An array of <code>MacroCommands</code> or subclasses of.
			 *
			 * @type {Array}
			 * @private
			 */
			subCommands: null,

			/**
			 * @override
			 *
			 * Initialize a <code>MacroCommand</code> instance.
			 */
			initialize: function()
			{
				this.subCommands = [];
				this.initializeMacroCommand();
			},

			/**
			 * @abstract
			 * Initialize the <code>MacroCommand</code>.
			 *
			 * <P>
			 * In your subclass, override this method to initialize the <code>MacroCommand</code>'s <i>subCommands</i>
			 * list with <code>Command</code> class references like this:
			 *
			 * <pre>
			 *    // Initialize MyMacroCommand
			 *    initializeMacroCommand: function()
			 *    {
			 *    	this.addSubCommand(FirstCommand);
			 *      this.addSubCommand(SecondCommand);
			 *      this.addSubCommand(ThirdCommand);
			 *    }
			 * </pre>
			 *
			 * <P>
			 * Note that <i>subCommands</i> may be any <code>Command</code> implementor.
			 *
			 * <P>
			 * In the JavaScript version it means that it only needs to implement an execute method and inherits from
			 * <code>Notifier</code>.
			 */
			initializeMacroCommand: function(){},

			/**
			 * Add an entry to <i>subCommands</i> list.
			 *
			 * <P>
			 * The <i>subCommands</i> will be called in First In/First Out (FIFO) order.
			 *
			 * @param {String} classPath
			 * 		Classpath of the <code>Command</code> class to add.
			 */
			addSubCommand: function( classPath )
			{
				this.subCommands.push( classPath );
			},

			/**
			 * Execute this <code>MacroCommand</code>'s <i>SubCommands</i>.
			 *
			 * <P>
			 * The <i>subCommands</i> will be called in First In/First Out (FIFO) order.
			 *
			 * @param {Notification} note
			 * 		The <code>Notification</code> object to be passed to each entry	of <i>subCommands</i> list.
			 */
			execute: function( note )
			{
				var len/*Number*/ = this.subCommands.length;
				for( var i/*Number*/=0; i<len; i++ )
				{
					var classPath/*String*/ = this.subCommands[i];
					var commandClass/*Function*/ = Objs(classPath);
					if( !commandClass )
						continue;

					var commandInstance/*Command*/ = new commandClass();
					commandInstance.setModel( this.model );
					commandInstance.setView( this.view );
					commandInstance.execute( note );
				}
			}
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Notification = Objs("truemvc.observer.Notification");

	/**
	 * @class truemvc.mediator.Mediator
	 * @classDescription
	 * <P>A base <code>Mediator</code> implementation.
	 * 
	 * <P>
	 * Typically, a <code>Mediator</code> will be written to serve one specific control or group controls and so, will
	 * not have a need to be dynamically named.
	 * 
	 * @see truemvc.observer.Notification
	 * @extends truemvc.observer.Notifier Notifier
	 * 
	 * @constructor
	 */
	var Mediator = Objs
	(
		"truemvc.mediator.Mediator",
		"truemvc.observer.Notifier",
		{
			/**
			 * The <code>View</code> this <code>Mediator</code> has to know.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,

			/**
			 * The <code>Model</code> this <code>Mediator</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * The name of the <code>Mediator</code>.
			 * 
			 * @type {String}
			 * @private
			 */
			mediatorName: null,
			
			/**
			 * The <code>Mediator</code>'s view component.
			 * 
			 * @type {Object}
			 * @private
			 */
			viewComponent: null,
			
			/**
			 * @override
			 *
			 * Initialize a <code>Mediator</code> instance.
			 *
			 * @param {String} mediatorName
			 * 		The name of the <code>Mediator</code>.
			 *
			 * @param {Object} viewComponent
			 * 		The <code>Mediator</code>'s view component.
			 *
			 */
			initialize: function( mediatorName, viewComponent )
			{
				this.mediatorName = (mediatorName != null) ? mediatorName : Mediator.NAME;
				this.viewComponent = viewComponent;
			},

			/**
			 * Get the <code>View</code> instance for this <code>Mediator</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Mediator</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Mediator</code>.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Mediator</code>.
			 */
			setView: function( view )
			{
				this.view = view;
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Mediator</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Mediator</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Mediator</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Mediator</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * @override
			 */
			sendNotification: function( name, body, type )
			{
				var note/*Notification*/ = new Notification( name, body, type );
				this.view.notifyObservers(note);
			},

			/**
			 * List the <code>Notification</code> names this <code>Mediator</code> is interested in being notified of.
			 *
			 * @return {Array}
			 * 		The list of notifications names in which is interested the
			 * 		<code>Mediator</code>.
			 */
			listNotificationInterests: function()
			{
				return [];
			},
			
			/**
			 * Get the name of the <code>Mediator</code>.
			 *
			 * @return {String}
			 * 		The <code>Mediator</code> name.
			 */
			getMediatorName: function()
			{
				return this.mediatorName;
			},
			
			/**
			 * Get the <code>Mediator</code>'s view component.
			 *
			 * @return {Object}
			 * 		The view component.
			 */
			getViewComponent: function()
			{
				return this.viewComponent;
			},
			
			/**
			 * Set the <code>Mediator</code>'s view component.
			 *
			 * @param {Object} viewComponent
			 * 		The view component.
			 */
			setViewComponent: function( viewComponent )
			{
				this.viewComponent = viewComponent;
			},
			
			/**
			 * Handle <code>Notification</code>s.
			 *
			 * <P>
			 * Typically this will be handled in a switch statement, with one 'case' entry per <code>Notification</code>
			 * the <code>Mediator</code> is interested in.
			 *
			 * @param {Notification} note
			 * 		The notification instance to be handled.
			 */
			handleNotification: function( note ){},
			
			/**
			 * Called by the View when the Mediator is registered.
			 *
			 * <P>
			 * This method is usually overridden as needed by the subclass.
			 */
			onRegister: function(){},
			
			/**
			 * Called by the View when the Mediator is removed.
			 *
			 * <P>
			 * This method is usually overridden as needed by the subclass.
			 */
			onRemove: function(){}
		}
	);
	
	/**
	 * Default name of the <code>Mediator</code>.
	 * 
	 * @type {String}
	 * @constant
	 */
	Mediator.NAME = "Mediator";
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.proxy.Proxy
	 *
	 * @classDescription
	 * The base <code>Proxy</code> class.
	 * 
	 * <P>
	 * In TrueMVC, <code>Proxy</code> classes are used to manage parts of the application's data model.
	 *
	 * <P>
	 * A <code>Proxy</code> might simply manage a reference to a local data object, in which case interacting with it
	 * might involve setting and getting of its data in synchronous fashion.
	 *
	 * <P>
	 * <code>Proxy</code> classes are also used to encapsulate the application's interaction with remote services to
	 * store or retrieve data, in which case, we adopt an asynchronous idiom; setting data (or calling a method) on the
	 * <code>Proxy</code> and listening for a <code>Notification</code> to be sent when the <code>Proxy</code> has
	 * retrieved the data from the service.
	 *
	 * @see truemvc.mvc.Model
	 *
	 * @extends truemvc.observer.Notifier Notifier
	 * 
	 * @constructor
	 */
	var Proxy = Objs
	( 
		"truemvc.proxy.Proxy",
		"truemvc.observer.Notifier",
		{

			/**
			 * The <code>Model</code> this <code>Proxy</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * The data object controlled by the <code>Proxy</code>.
			 *
			 * @type {Object}
			 * @private
			 */
			data: null,
			
			/**
			 * The name of the <code>Proxy</code>.
			 * 
			 * @type {String}
			 * @private
			 */
			proxyName: null,
			
			/**
			 * @override
			 *
			 * Initialize a <code>Proxy</code> instance.
			 *
			 * @param {String} proxyName
			 * 		The name of the <code>Proxy</code>.
			 *
			 * @param {Object} data
			 * 		An initial data object to be held by the <code>Proxy</code>.
			 */
			initialize: function( proxyName, data )
			{
				this.proxyName = (proxyName != null) ? proxyName : Proxy.NAME;
				this.data = data;
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Proxy</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Proxy</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Proxy</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Proxy</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * @override
			 */
			sendNotification: function( name, body, type )
			{
				var note/*Notification*/ = new Notification( name, body, type );
				this.model.notifyObservers(note);
			},

			/**
			 * Gets the proxyName.
			 *
			 * @return {String}
			 * 		The name of the proxy.
			 */
			getProxyName: function()
			{
				return this.proxyName;
			},
			
			/**
			 * Sets the data object.
			 *
			 * @param {Object} data
			 * 		The data to set.
			 */
			setData: function( data )
			{
				this.data = data;
			},
			
			/**
			 * Gets the data.
			 *
			 * @return {Object}
			 * 		The data held in the <code>Proxy.
			 */
			getData: function()
			{
				return this.data;
			},
			
			/**
			 * Called by the Model when the <code>Proxy</code> is registered.
			 *
			 * <P>
			 * This method is usually overridden as needed by the subclass.
			 */
			onRegister: function(){},
			
			/**
			 * Called by the Model when the <code>Proxy</code> is removed.
			 *
			 * <P>This method is usually overridden as needed by the subclass.
			 */
			onRemove: function(){}
		}
	);

	/**
	 * The default name of the <code>Proxy</code>
	 * 
	 * @type {String}
	 * @constant
	 */
	Proxy.NAME = "Proxy";
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Observer = Objs("truemvc.observer.Observer");
	var Observable = Objs("truemvc.observer.Observable");
	var Mediator = Objs("truemvc.mediator.Mediator");

	/**
	 * @class truemvc.mvc.View
	 *
	 * @classDescription
	 * The <code>View</code> class in TrueMVC.
	 *
	 * <P>
	 * In TrueMVC, the <code>View</code> class assumes these responsibilities:
	 *
	 * <UL>
	 * 	<LI>Maintain a cache of <code>Mediator</code> instances.
	 * 	<LI>Provide methods for registering, retrieving, and removing <code>Mediator</code>s.
	 * 	<LI>Notifiying <code>Mediator</code>s when they are registered or removed.
	 * 	<LI>Managing the <code>Observer</code> lists for each <code>Notification</code> in the application.
	 * 	<LI>Providing a method for attaching <code>Observer</code>s to an <code>Notification</code>'s
	 * 	<code>Observer</code> list.
	 * 	<LI>Providing a method for broadcasting a <code>Notification</code>.
	 * 	<LI>Notifying the <code>Observer</code>s of a given <code>Notification</code> when it broadcasts.
	 *
	 * @see truemvc.mediator.Mediator
	 * @see truemvc.observer.Observer
	 * @see truemvc.observer.Notification
	 *
	 * @constructor
	 */
	var View = Objs
	(
		"truemvc.mvc.View",
		{

			/**
			 * The <code>Model</code> this <code>View</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * @private
			 *
			 * Mapping of <code>Mediator</code> names to <code>Mediator</code> instances.
			 *
			 * @type {Object}
			 */
			mediatorMap: null,

			/**
			 * @private
			 *
			 * Observable composite object for the <code>View></code>.
			 *
			 * @type {Observable}
			 */
			observable: null,

			/**
			 * Initialize a <code>View</code> instance.
			 */
			initialize: function()
			{
				this.mediatorMap = {};
				this.observable = new Observable();
			},

			/**
			 * Get the <code>Model</code> instance for this <code>View</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>View</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>View</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>View</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * Register an <code>Observer</code> to be notified of <code>Notifications</code> with a given name.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code>s to notify this <code>Observer</code> of.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> to register.
			 */
			registerObserver: function( name, observer )
			{
				this.observable.registerObserver( name, observer );
			},

			/**
			 * Notify the <code>Observer</code>s for a particular <code>Notification</code>.
			 *
			 * <P>
			 * All previously attached <code>Observer</code>s for this <code>Notification</code>'s list are notified and
			 * are passed a reference to the <code>Notification</code> in the order in which they were registered.
			 *
			 * @param {Notification} note
			 * 		The <code>Notification</code> to notify <code>Observer</code>s of.
			 */
			notifyObservers: function( note )
			{
				this.observable.notifyObservers( note );
			},

			/**
			 * Remove the <code>Observer</code> for a given <i>notifyContext</i> from an <code>Observer</code> list for
			 * a given <code>Notification</code> name.
			 *
			 * @param {String} name
			 * 		Which <code>Observer</code> list to remove from.
			 *
			 * @param {Object} notifyContext
			 * 		Remove the <code>Observer</code> with this object as its <i>notifyContext</i>.
			 */
			removeObserver: function( name, notifyContext )
			{
				this.observable.removeObserver( name, notifyContext );
			},

			/**
			 * Register an <code>IMediator</code> instance with the <code>View</code>.
			 *
			 * <P>
			 * Registers the <code>IMediator</code> so that it can be retrieved by name, and further interrogates the
			 * <code>IMediator</code> for its <code>INotification</code> interests.
			 *
			 * <P>
			 * If the <code>IMediator</code> returns any <code>INotification</code> names to be notified about, an
			 * <code>Observer</code> is created to encapsulate the <code>IMediator</code> instance's
			 * <code>handleNotification</code> method and register it as an <code>Observer</code> for all
			 * <code>INotification</code>s the <code>IMediator</code> is interested in.
			 *
			 * @param {Mediator} mediator
			 * 		A reference to the <code>Mediator</code> instance.
			 */
			registerMediator: function( mediator )
			{
				var name/*String*/ = mediator.getMediatorName();

				// Do not allow re-registration (you must removeMediator first)
				if( this.hasMediator( mediatorName ) )
					return;

				// Register the Mediator for retrieval by name
				this.mediatorMap[name] = mediator;

				// Register Mediator as an observer for each of its notification interests
				var interests/*Array*/ = mediator.listNotificationInterests();
				var len/*Number*/ = interests.length;
				if( len>0 )
				{
			    	// Register Mediator as Observer for its list of Notification interests
					var observer/*Observer*/ = new Observer( mediator.handleNotification, mediator );
					for( var i/*Number*/=0; i<len; i++ )
						this.registerObserver( interests[i], observer );
				}

				mediator.setModel(this.model);
				mediator.setView(this);
				mediator.onRegister();
			},

			/**
			 * Retrieve a <code>Mediator</code> from the <code>View</code>.
			 *
			 * @param {String} mediatorName
			 *		The name of the <code>IMediator</code> instance to retrieve.
			 *
			 * @return {Mediator}
			 *		The <code>Mediator</code> instance previously registered with the given <i>mediatorName</i> or an
			 *		explicit <code>null</code> if it doesn't exists.
			 */
			retrieveMediator: function( mediatorName )
			{
				//Return a strict null when the mediator doesn't exist
				if( !this.hasMediator(mediatorName) )
					return null;

				return this.mediatorMap[mediatorName];
			},

			/**
			 * Check if a <code>Mediator</code> is registered or not.
			 *
			 * @param {String} mediatorName
			 *		Name of the <code>Mediator</code> instance to verify the existence of its registration.
			 *
			 * @return {Boolean}
			 *		A <code>Mediator</code> is registered with the given <i>mediatorName</i>.
			 */
			hasMediator: function( mediatorName )
			{
				return this.mediatorMap[mediatorName] instanceof Mediator;
			},

			/**
			 * Remove a <code>Mediator</code> from the <code>View</code>.
			 *
			 * @param {String} mediatorName
			 *		Name of the <code>IMediator</code> instance to be removed.
			 *
			 * @return {Mediator}
			 *		The <code>Mediator</code> that was removed from the <code>View</code> or a strict <code>null</null>
			 *		if the <code>Mediator</code> didn't exist.
			 */
			removeMediator: function( mediatorName )
			{
				//Return a strict null when the mediator doesn't exist
				if( !this.hasMediator(mediatorName) )
					return null;

				var mediator/*Mediator*/ = this.mediatorMap[mediatorName];
				var interests/*Array*/ = mediator.listNotificationInterests();
				var i/*Number*/ = interests.length;
				while( i-- )
					this.removeObserver( interests[i], mediator );

				delete this.mediatorMap[mediatorName];
				mediator.onRemove();
				mediator.setModel(null);
				mediator.setView(null);
				return mediator;
			},


			/**
			 * Remove each registered <code>Mediator</code>.
			 *
			 * <P>
			 * Rem: Call this to mark the Mediator as available for garbage collection, as it will remove itself
			 * for being referenced in each <code>Mediator</code> previously registered with it.
			 */
			removeAllProxies: function()
			{
				//Make a copy of the proxyMap to be sure it will not be modified during the loop.
				var arr/*Array*/ = [];
				for( var mediatorName/*String*/ in this.mediatorMap )
					arr.push(mediatorName);

				var i/*Number*/ = arr.length;
				while( i-- )
					this.removeMediator(arr[i]);
			}
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Observer = Objs("truemvc.observer.Observer");
	var Observable = Objs("truemvc.observer.Observable");
	var Proxy = Objs("truemvc.proxy.Proxy");

	/**
	 * @class truemvc.mvc.Model
	 *
	 * @classDescription
	 * The <code>Model</code> class for TrueMVC.
	 *
	 * <P>
	 * In TrueMVC, the <code>Model</code> class provides access to model objects (Proxies) by named lookup.
	 *
	 * <P>
	 * The <code>Model</code> assumes these responsibilities:
	 *
	 * <UL>
	 * 	<LI>Maintain a cache of <code>Proxy</code> instances.
	 * 	<LI>Provide methods for registering, retrieving, and removing <code>Proxy</code> instances.
	 *
	 * <P>Your application must register <code>Proxy</code> instances with the <code>Model</code>. Typically, you use a
	 * <code>Command</code> to create and register <code>Proxy</code> instances once the <code>Facade</code> has
	 * initialized the core actors.
	 *
	 * @see truemvc.proxy.Proxy
	 *
	 * @constructor
	 */
	var Model = Objs
	(
		"truemvc.mvc.Model",
		{
			/**
			 * HashTable of <code>Proxy</code> instances registered with the <code>Model</code>.
			 *
			 * @type {Object}
			 * @private
			 */
			proxyMap: null,

			/**
			 * @private
			 *
			 * Observable composite object for the <code>View></code>.
			 *
			 * @type {Observable}
			 */
			observable: null,

			/**
			 * Initialize a <code>Model</code> instance.
			 */
			initialize: function()
			{
				this.proxyMap = {};
				this.observable = new Observable();
			},

			/**
			 * Register an <code>Observer</code> to be notified of <code>Notifications</code> with a given name.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code>s to notify this <code>Observer</code> of.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> to register.
			 */
			registerObserver: function( name, observer )
			{
				this.observable.registerObserver( name, observer );
			},

			/**
			 * Notify the <code>Observer</code>s for a particular <code>Notification</code>.
			 *
			 * <P>
			 * All previously attached <code>Observer</code>s for this <code>Notification</code>'s list are notified and
			 * are passed a reference to the <code>Notification</code> in the order in which they were registered.
			 *
			 * @param {Notification} note
			 * 		The <code>Notification</code> to notify <code>Observer</code>s of.
			 */
			notifyObservers: function( note )
			{
				this.observable.notifyObservers( note );
			},

			/**
			 * Remove the <code>Observer</code> for a given <i>notifyContext</i> from an <code>Observer</code> list for
			 * a given <code>Notification</code> name.
			 *
			 * @param {String} name
			 * 		Which <code>Observer</code> list to remove from.
			 *
			 * @param {Object} notifyContext
			 * 		Remove the <code>Observer</code> with this object as its <i>notifyContext</i>.
			 */
			removeObserver: function( name, notifyContext )
			{
				this.observable.removeObserver( name, notifyContext );
			},

			//FIXME Implement removeAllObservers

			/**
			 * Register a <code>Proxy</code> with the <code>Model</code>.
			 *
			 * @param {Proxy} proxy
			 *		A <code>Proxy</code> to be held by the <code>Model</code>.
			 */
			registerProxy: function( proxy )
			{
				//TODO Why authorizing re-registration here?
 				this.proxyMap[proxy.getProxyName()] = proxy;

				proxy.setModel(this);
				proxy.onRegister();
			},

			/**
			 * Retrieve an <code>IProxy</code> from the <code>Model</code>.
			 *
			 * @param {String} proxyName
			 *		The name of the <code>Proxy</code> to retrieve.
			 *
			 * @return {Proxy}
			 *		The <code>Proxy</code> instance previously registered with the given <code>proxyName</code> or an
			 *		explicit <code>null</code> if it doesn't exists.
			 */
			retrieveProxy: function( proxyName )
			{
				//Return a strict null when the proxy doesn't exist
				if( !this.hasProxy(proxyName) )
					return null;

				return this.proxyMap[proxyName];
			},

			/**
			 * Check if a <code>Proxy</code> is registered.
			 *
			 * @param {String} proxyName
			 *		The name of the <code>Proxy</code> to verify the existence of its registration.
			 *
			 * @return {Boolean}
			 *		A Proxy is currently registered with the given <code>proxyName</code>.
			 */
			hasProxy: function( proxyName )
			{
				return this.proxyMap[proxyName] instanceof Proxy;
			},

			/**
			 * Remove a <code>Proxy</code> from the <code>Model</code>.
			 *
			 * @param {String} proxyName
			 *		The name of the <code>Proxy</code> instance to be removed.
			 *
			 * @return {Proxy}
			 *		The <code>Proxy</code> that was removed from the <code>Model</code> or an explicit <code>null</null>
			 *		if the <code>Proxy</code> didn't exist.
			 */
			removeProxy: function( proxyName )
			{
				if( !this.hasProxy(proxyName) )
					return null;

				var proxy/*Proxy*/ = this.proxyMap[proxyName];
				delete this.proxyMap[proxyName];
				proxy.onRemove();
				proxy.setModel(null);
				return proxy;
			},

			/**
			 * Remove each registered <code>Proxy</code>.
			 *
			 * <P>
			 * Rem: Call this to mark the Model as available for garbage collection, as it will remove itself
			 * for being referenced in each <code>Proxy</code> previously registered with it.
			 */
			removeAllProxies: function()
			{
				//Make a copy of the proxyMap to be sure it will not be modified during the loop.
				var arr/*Array*/ = [];
				for( var proxyName/*String*/ in this.proxyMap )
					arr.push(proxyName);

				var i/*Number*/ = arr.length;
				while( i-- )
					this.removeProxy(arr[i]);
			}
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Observer = Objs("truemvc.observer.Observer");
	var View = Objs("truemvc.mvc.View");

	/**
	 * @class truemvc.mvc.Controller
	 *
	 * @classDescription
	 * The <code>Controller</code> class for TrueMVC.
	 *
	 * <P>
	 * In TrueMVC, the <code>Controller</code> class follows the 'Command and Controller' strategy, and assumes these
	 * responsibilities:
	 *
	 * <UL>
	 * <LI>Remembering which <code>Command</code>s or <code>MacroCommand</code>s are intended to handle which
	 * <code>Notification</code>s.
	 *
	 * <LI>Registering itself as an <code>Observer</code> with the <code>View</code> for each <code>Notification</code>
	 * that it has a <code>Command</code> or <code>MacroCommand</code> mapping for.
	 *
	 * <LI>Creating a new instance of the proper <code>Command</code> or <code>MacroCommand</code> to handle a
	 * given <code>Notification</code> when notified by the <code>View</code>.
	 *
	 * <LI>Calling the <code>Command</code>'s or <code>MacroCommand</code>'s  <code>execute</code> method, passing
	 * in the <code>Notification</code>.
	 *
	 * <P>
	 * Your application must register <code>ICommand</code>s with the <code>Controller</code>.
	 *
	 * <P>
	 * The simplest way is to subclass </code>Facade</code>, and use its <code>initializeController</code> method to add
	 * your registrations.
	 *
	 * @see truemvc.View
	 * @see truemvc.observer.Observer
	 * @see truemvc.observer.Notification
	 * @see truemvc.command.Command
	 * @see truemvc.command.MacroCommand
	 *
	 * @constructor
	 */
	var Controller = Objs
	(
		"truemvc.mvc.Controller",
		{
			/**
			 * The <code>View</code> this <code>Controller</code> has to know.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,

			/**
			 * The <code>Model</code> this <code>Controller</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * Mapping of <code>Notification<code> names to <code>Command</code> class references.
			 *
			 * @type {Object}
			 * @private
			 */
			commandMap: null,

			/**
			 * Initialize a <code>Controller</code> instance.
			 */
			initialize: function()
			{
				this.commandMap = {};
			},

			/**
			 * Get the <code>View</code> instance for this <code>Controller</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Controller</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Controller</code>.
			 *
			 * <P>
			 * Setting it to <code>null</code> will not make the <code>View</code> instance available for garbage as it
			 * is passed as reference to commands. Be sure to unregister all commands to make it available for garbage.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Controller</code>.
			 */
			setView: function( view )
			{
				this.view = view;
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Controller</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Controller</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Controller</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Controller</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * If a <code>Command</code> has previously been registered to handle the given <code>Notification</code>,
			 * then it is executed.
			 *
			 * @param {Notification} note
			 * 		A <code>Notification</code>.
			 */
			executeCommand: function( note )
			{
				var notificationName/*String*/ = note.getName();
				if( this.hasCommand(notificationName) )
				{
					var classPath/*String*/ = this.commandMap[notificationName];
					var commandClass/*Function*/ = Objs(classPath);
					var command/*Command*/ = new commandClass();
					command.setModel(this.model);
					command.setView(this.view);
					command.execute(note);
				}
			},

			/**
			 * Register a particular <code>Command</code> class as the handler for a particular
			 * <code>Notification</code>.
			 *
			 * <P>
			 * If a <code>Command</code> has already been registered to handle <code>Notification</code>s with this
			 * name, it is no longer used, the new <code>Command</code> is used instead.
			 *
			 * The <code>Observer</code> for the new <code>Command</code> is only created if this is the first time a
			 * <code>Command</code> has been registered for this <code>Notification</code> name.
			 *
			 * @param {String} notificationName
			 * 		The name of the <code>Notification</code>.
			 *
			 * @param {String} classPath
			 * 		The classpath of the <code>Command</code> class to register.
			 */
			registerCommand: function( notificationName, classPath )
			{
				//TODO Why authorizing re-registration here?
				if( !this.hasCommand(notificationName) )
				{
					if( this.view )
					{
						var observer/*Observer*/ = new Observer( this.executeCommand, this );
						this.view.registerObserver( notificationName, observer );
					}
				}

				this.commandMap[notificationName] = classPath;
			},

			/**
			 * Check if a <code>Command</code> is registered for a given <code>Notification</code>.
			 *
			 * <P>
			 * Rem: Do not verify if the class object associated to the command classpath exists.
			 *
			 * @param {String} notificationName
			 * 		The name of the <code>Notification</code> to verify the	existence of its registration.
			 *
			 * @return {Boolean}
			 * 		A <code>Command</code> is currently registered for the given <code>notificationName</code>.
			 */
			hasCommand: function( notificationName )
			{
				return typeof this.commandMap[notificationName] == "string";
			},

			/**
			 * Remove a previously registered <code>Command</code> or <code>MacroCommand</code> to
			 * <code>Notification</code> mapping.
			 *
			 * @param {String} notificationName
			 * 		The name of the <code>Notification</code> to remove the	<code>Command</code> or
			 * 		<code>MacroCommand</code> mapping for.
			 */
			removeCommand: function( notificationName )
			{
				if( this.hasCommand(notificationName) )
				{
					if( this.view )
						this.view.removeObserver( notificationName, this );

					delete this.commandMap[notificationName];
				}
			},

			/**
			 * Remove all registered <code>Command</code>s or <code>MacroCommand</code>s to
			 * <code>Notification</code> mapping.
			 *
			 * <P>
			 * Rem: Call this to mark the Controller as available for garbage collection, as it will remove itself
			 * for being referenced in <code>Observer</code>s object detained by the view.
			 */
			removeAllCommands: function()
			{
				//Make a copy of the commandMap to be sure it will not be modified during the loop.
				var arr/*Array*/ = [];
				for( var notificationName/*String*/ in this.commandMap )
					arr.push(notificationName);

				var i/*Number*/ = arr.length;
				while( i-- )
					this.removeCommand(arr[i]);
			}
		}
	);
}
/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Model = Objs("truemvc.mvc.Model");
	var View = Objs("truemvc.mvc.View");
	var Controller = Objs("truemvc.mvc.Controller");
	var Notification = Objs("truemvc.observer.Notification");
	
	/**
	 * @class Facade
	 *
	 * @classDescription
	 * The base <code>Facade</code> implementation for each core.
	 * 
	 * <P>
	 * In TrueMVC, the <code>Facade</code> class assumes these responsibilities:
	 * 
	 * <UL>
	 * 		<LI>Initializing the <code>Model</code>, <code>View</code> and <code>Controller</code> core actors.
	 * 		<LI>Providing all the applicable methods of the <code>Model</code>, <code>View</code>, &
	 * 		<code>Controller</code> instances.
	 * 		<LI>Providing a single point of contact to the application for registering <code>Command</code>s and
	 * 		notifying <code>Observer</code>s.
	 * 
	 * @see truemvc.mvc.Controller
	 * @see truemvc.mvc.Model
	 * @see truemvc.mvc.View
	 * @see truemvc.observer.Notification
	 * @see truemvc.mediator.Mediator
	 * @see truemvc.proxy.Proxy
	 * @see truemvc.command.Command
	 * @see truemvc.command.MacroCommand
	 *
	 * @constructor
	 */
	var Facade = Objs
	(
		"truemvc.facade.Facade",
		{
			/**
			 * The <code>View</code> for this <code>Facade</code>.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,
			
			/**
			 * The <code>Model</code> for this <code>Facade</code>.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,
			
			/**
			 * The <code>Controller</code> for this <code>Facade</code>.
			 *
			 * @type {Controller}
			 * @private
			 */
			controller: null,
			
			/**
			 * Initialize a <code>Facade</code> instance.
			 *
			 * Don't call it in your subclass for a custom initialization of the <code>Facade</code> that does not
			 * require the <code>Facade</code> to be automatically initialized with <code>initializeFacade</code>.
			 */
			initialize: function()
			{
				this.initializeFacade();
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Facade</code>.
			 *
			 * @return {Model}
			 * 		The <code>View</code> instance set for this <code>Facade</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Facade</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Facade</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * Get the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Command</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Command</code>.
			 */
			setView: function( view )
			{
				this.view = view;
			},

			/**
			 * Get the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @return {Controller}
			 * 		The <code>Controller</code> instance set for this <code>Command</code>.
			 */
			getController: function()
			{
				return this.controller;
			},

			/**
			 * Set the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @param {Controller} controller
			 * 		The <code>Model</code> instance to set for this <code>Command</code>.
			 */
			setController: function( controller )
			{
				this.controller = controller;
			},

			/**
			 * Automatically initialize the <code>Facade</code> instance with its <code>Model</code>, <code>View</code>,
			 * <code>Controller</code>.
			 *
			 * <P>
			 * Called automatically by the constructor.
			 */
			initializeFacade: function()
			{
				var model/*Model*/ = new Model();
				var view/*View*/ = new View();
				var controller/*Controller*/ = new Controller();

				this.setModel( model );
				this.setView( view );
				this.setController( controller );

				this.initializeModel();
				this.initializeView();
				this.initializeController();
			},

			/**
			 * Initialize the <code>Model</code>.
			 *
			 * <P>
			 * Called by the <code>initializeFacade</code> method.
			 *
			 * Override this method in your subclass of <code>Facade</code>  if one or both of the following are true:
			 * <UL>
			 * 		<LI> You wish to initialize a different <code>Model</code>.
			 * 		<LI> You have <code>Proxy</code>s to register with the Model that do not retrieve a reference to the
			 * 		Facade at construction time.
			 *
			 * If you don't want to initialize a different <code>Model</code>, call
			 * <code>$super.initializeModel()</code> at the beginning of your method, then register <code>Proxy</code>s.
			 *
			 * <P>
			 * Note: This method is <i>rarely</i> overridden; in practice you are more likely to use a
			 * <code>Command</code> to create and register <code>Proxy</code>s with the <code>Model</code>, since
			 * <code>Proxy</code>s with mutable data will likely need to send <code>Notification</code>s and thus will
			 * likely want to fetch a reference to the <code>Facade</code> during their construction.
			 */
			initializeModel: function()
			{
				//FIXME anny previously registered model has to be cleanly removed (dedicated method)

				//In case the Model is set after the Controller
				if( this.controller )
					this.controller.setModel(this.model);

				//In case the Model is set after the View
				if( this.view )
					this.view.setModel(this.model);
			},

			/**
			 * Initialize the <code>View</code>.
			 *
			 * <P>
			 * Called by the <code>initializeFacade</code> method.
			 *
			 * Override this method in your subclass of <code>Facade</code> if one or both of the following are true:
			 * <UL>
			 * 		<LI> You wish to initialize a different <code>View</code>.
			 * 		<LI> You have <code>Observers</code> to register with the <code>View</code>
			 *
			 * If you don't want to initialize a different <code>View</code>, call <code>super.initializeView()</code>
			 * at the beginning of your method, then register <code>Mediator</code> instances.
			 *
			 * <P>
			 * Note: This method is <i>rarely</i> overridden; in practice you are more likely to use a
			 * <code>Command</code> to create and register <code>Mediator</code>s with the <code>View</code>, since
			 * <code>Mediator</code> instances will need to send <code>Notification</code>s and thus will likely want
			 * to fetch a reference to the <code>Facade</code> during their construction.
			 */
			initializeView: function()
			{
				//FIXME anny previously registered view has to be cleanly removed (dedicated method)

				//In case the View is set after the Model
				if( this.model )
					this.view.setModel(this.model);

				//In case the View is set after the Controller
				if( this.controller )
					this.controller.setView(this.view);
			},

			/**
			 * Initialize the <code>Controller</code>.
			 *
			 * <P>
			 * Called by the <code>initializeFacade</code> method.
			 *
			 * Override this method in your subclass of <code>Facade</code> if one or both of the following are true:
			 * <UL>
			 * 		<LI> You wish to initialize a different <code>Controller</code>.
			 * 		<LI> You have <code>Commands</code> to register with the <code>Controller</code> at startup.</code>.
			 *
			 * If you don't want to initialize a different <code>IController</code>, call
			 * <code>$super.initializeController()</code> at the beginning of your method, then register
			 * <code>Command</code>s.
			 */
			initializeController: function()
			{
				//FIXME anny previously registered controller has to be cleanly removed (dedicated method)

				//In case the Controller is set after the View
				if( this.view )
					this.controller.setView(this.view);

				//In case the Controller is set after the Model
				if( this.model )
					this.controller.setModel(this.model);
			},

			/**
			 * Register a <code>Command</code> with the <code>Controller</code> by <code>Notification</code> name.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code> to associate the <code>Command</code> with.
			 *
			 * @param {Function} commandClassRef
			 * 		A reference to the Class of the <code>Command</code>.
			 */
			registerCommand: function( name, commandClassRef )
			{
				this.controller.registerCommand( name, commandClassRef );
			},
			
			/**
			 * Remove a previously registered <code>Command</code> to <code>Notification</code> mapping from the
			 * <code>Controller</code>.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code> to remove the <code>Command</code> mapping for.
			 */
			removeCommand: function( name )
			{
				this.controller.removeCommand(name);
			},

			/**
			 * Remove all previously registered <code>Command</code>s or <code>MacroCommand</code>s to
			 * <code>Notification</code> mapping in a one time pass.
			 *
			 * <P>
			 * Rem: Call this to mark the Controller as available for garbage collection, as it will remove itself
			 * for being referenced in Observers object detained by the view.
			 */
			removeAllCommands: function()
			{
				this.controller.removeAllCommands();
			},
			
			/**
			 * Check if a <code>Command</code> is registered for a given <code>Notification</code>.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code> to verify for the existence of a <code>Command</code>
			 * 		mapping for.
			 *
			 * @return {Boolean}
			 * 		A <code>Command</code> is currently registered for the given <i>name</i>.
			 */
			hasCommand: function( name )
			{
				return this.controller.hasCommand(name);
			},
			
			/**
			 * Register a <code>Proxy</code> with the <code>Model</code> by name.
			 *
			 * @param proxy {Proxy}
			 * 		The <code>Proxy</code> instance to be registered with the <code>Model</code>.
			 */
			registerProxy: function( proxy )
			{
				this.model.registerProxy( proxy );
			},
			
			/**
			 * Retrieve a <code>Proxy</code> from the <code>Model</code> by name.
			 *
			 * @param {String} proxyName
			 * 		The name of the <code>Proxy</code> to be retrieved.
			 *
			 * @return {Proxy}
			 * 		The <code>Proxy</code> instance previously registered with the given <i>proxyName</i>.
			 */
			retrieveProxy: function( proxyName )
			{
				return this.model.retrieveProxy(proxyName);
			},
			
			/**
			 * Remove an <code>Proxy</code> from the <code>Model</code> by name.
			 *
			 * @param {String} proxyName
			 * 		The <code>Proxy</code> to remove from the <code>Model</code>.
			 *
			 * @return {Proxy}
			 * 		The <code>Proxy</code> that was removed from the <code>Model</code>.
			 */
			removeProxy: function( proxyName )
			{
				return this.model.removeProxy(proxyName);
			},

			/**
			 * Remove each registered <code>Proxy</code> from the <code>Model</code>.
			 *
			 * <P>
			 * Rem: Call this to mark the Model as available for garbage collection, as it will remove itself
			 * for being referenced in each <code>Proxy</code> previously registered with it.
			 */
			removeAllProxies: function()
			{
				this.model.removeAllProxies();
			},

			/**
			 * Check if a <code>Proxy</code> is registered.
			 *
			 * @param {String} proxyName
			 * 		The <code>Proxy</code> to verify the existence of a registration with the <code>Model</code>.
			 *
			 * @return {Boolean}
			 * 		A <code>Proxy</code> is currently registered with the given <i>proxyName</i>.
			 */
			hasProxy: function( proxyName )
			{
				return this.model.hasProxy(proxyName);
			},
			
			/**
			 * Register a <code>Mediator</code> with the <code>View</code>.
			 *
			 * @param {Mediator} mediator
			 * 		A reference to the <code>Mediator</code>.
			 */
			registerMediator: function( mediator )
			{
				this.view.registerMediator( mediator );
			},
			
			/**
			 * Retrieve an <code>Mediator</code> from the <code>View</code>.
			 *
			 * @param {String} mediatorName
			 * 		The name of the registered <code>Mediator</code> to retrieve.
			 *
			 * @return {Mediator}
			 * 		The <code>Mediator</code> previously registered with the given <i>mediatorName</i>.
			 */
			retrieveMediator: function( mediatorName )
			{
				return this.view.retrieveMediator(mediatorName);
			},
			
			/**
			 * Remove an <code>Mediator</code> from the <code>View</code>.
			 *
			 * @param {String} mediatorName
			 * 		The name of the <code>Mediator</code> to be removed.
			 *
			 * @return {Mediator}
			 * 		The <code>Mediator</code> that was removed from the <code>View</code>.
			 */
			removeMediator: function( mediatorName )
			{
				return this.view.removeMediator(mediatorName);
			},
			
			/**
			 * Check if a <code>Mediator</code> is registered or not.
			 *
			 * @param {String} mediatorName
			 * 		The name of the <code>Mediator</code> to verify the existence of a registration for.
			 *
			 * @return {Boolean}
			 * 		A <code>Mediator</code> is registered with the given <i>mediatorName</i>.
			 */
			hasMediator: function( mediatorName )
			{
				return this.view.hasMediator(mediatorName);
			},
			
			/**
			 * Create and send a <code>Notification</code>.
			 *
			 * <P>Keeps us from having to construct new notification instances in our implementation code.
			 *
			 * @param {String} name
			 * 		The name of the notification to send.
			 *
			 * @param {Object} body
			 * 		The body of the notification to send.
			 *
			 * @param {String} type
			 * 		The type of the notification to send.
			 */
			sendNotification: function( name, body, type )
			{
				var note/*Notification*/ = new Notification( name, body, type );
				this.view.notifyObservers(note);
			}
		}
	);
}

if( typeof HideTrueMVC == "undefined" )
{
    var Observer = Objs("truemvc.observer.Observable");
    var Observer = Objs("truemvc.observer.Observer");
    var Notification = Objs("truemvc.observer.Notification");
    var View = Objs("truemvc.mvc.View");
    var Model = Objs("truemvc.mvc.Model");
    var Controller = Objs("truemvc.mvc.Controller");
    var Facade = Objs("truemvc.facade.Facade");
    var Notifier = Objs("truemvc.observer.Notifier");
    var Command = Objs("truemvc.command.Command");
    var MacroCommand = Objs("truemvc.command.MacroCommand");
    var Mediator = Objs("truemvc.mediator.Mediator");
    var Proxy = Objs("truemvc.proxy.Proxy");
}
        