/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Notification = Objs("truemvc.observer.Notification");

	/**
	 * @class truemvc.command.Command
	 *
	 * @classDescription
	 * A base <code>Command</code> implementation.
	 * 
	 * <P>
	 * Your subclass should override the <code>execute</code> method where your business logic will handle the
	 * <code>Notification</code>.
	 *
	 * <P>
	 * As in JavaScript there isn't interfaces, <code>Command</code> and <code>MacroCommand</code> cannot offer
	 * the guarantee to have the right signature. We could have inherited from a common <code>Command</code> class,
	 * but to avoid an unwanted complexity and to respect TrueMVC implementation, this is to the developer to take care
	 * to inherit from <code>Command</code> in its command and <code>MacroCommand</code> depending on his need.
	 * 
	 * @implements truemvc.interfaces.INotifier
	 *
	 * @constructor
	 */
	var Command = Objs
	(
		"truemvc.command.Command",
		{

			/**
			 * The <code>View</code> this <code>Command</code> has to know.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,

			/**
			 * The <code>Model</code> this <code>Command</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * The <code>Controller</code> this <code>Command</code> has to know.
			 *
			 * @type {Controller}
			 * @private
			 */
			controller: null,

			/**
			 * Get the <code>Model</code> instance for this <code>Command</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Command</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Command</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Command</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * Get the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Command</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Command</code>.
			 */
			setView: function( view )
			{
				this.view = view;
			},

			/**
			 * Get the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @return {Controller}
			 * 		The <code>Controller</code> instance set for this <code>Command</code>.
			 */
			getController: function()
			{
				return this.controller;
			},

			/**
			 * Set the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @param {Controller} controller
			 * 		The <code>Model</code> instance to set for this <code>Command</code>.
			 */
			setController: function( controller )
			{
				this.controller = controller;
			},

			/**
			 * @override
			 */
			sendNotification: function( name, body, type )
			{
				var notification/*Notification*/ = new Notification( name, body, type );
				this.view.notifyObservers(notification);
			},

			/**
			 * @abstract
			 *
			 * Fulfill the use-case initiated by the given <code>Notification</code>.
			 *
			 * <P>
			 * In the Command Pattern, an application use-case typically begins with some user action, which results in
			 * a <code>Notification</code> being broadcast, which is handled by business logic in the
			 * <code>execute</code> method of an <code>Command</code>.
			 *
			 * @param {Notification} notification
			 * 		The <code>Notification</code> to handle.
			 */
			execute: function( notification )
			{
				throw Error("Command.execute method is abstract and must be overridden.");
			}
		}
	);
}