/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Notification = Objs("truemvc.observer.Notification");

	/**
	 * @class truemvc.command.MacroCommand
	 *
	 * @classDescription
	 * A base <code>Command</code> implementation that executes other <code>Command</code>s.
	 *
	 * <P>
	 * A <code>MacroCommand</code> maintains an list of <code>Command</code> class references called <i>SubCommands</i>.
	 *
	 * <P>
	 * When <code>execute</code> is called, the <code>MacroCommand</code> instantiates and calls <code>execute</code> on
	 * each of its <i>SubCommands</i> turn. Each <i>SubCommand</i> will be passed a reference to the original
	 * <code>Notification</code> that was passed to the <code>MacroCommand</code>'s <code>execute</code> method.
	 *
	 * <P>
	 * Unlike <code>MacroCommand</code>, your subclass should not override <code>execute</code>, but instead, should
	 * override the <code>initializeMacroCommand</code> method, calling <code>addSubCommand</code> once for each
	 * <i>SubCommand</i> to be executed.
	 *
	 * <P>
	 * As in JavaScript there isn't interfaces, <code>MacroCommand</code> and <code>MacroCommand</code> cannot offer
	 * the guarantee to have the right signature. We could have inherited from a common <code>Command</code> class,
	 * but to avoid an unwanted complexity and to respect TrueMVC implementation, this is to the developer to take care
	 * to inherit from <code>MacroCommand</code> in its command and <code>MacroCommand</code> depending on his need.
	 *
	 * @implements truemvc.observer.INotifier
	 *
	 * @constructor
	 */
	var MacroCommand = Objs
	(
		"truemvc.command.MacroCommand",
		"truemvc.command.Command",
		{
			/**
			 * An array of <code>MacroCommands</code> or subclasses of.
			 *
			 * @type {Array}
			 * @private
			 */
			subCommands: null,

			/**
			 * @override
			 * @constructs
			 * Initialize a <code>MacroCommand</code> instance.
			 */
			initialize: function()
			{
				this.subCommands = [];
				this.initializeMacroCommand();
			},

			/**
			 * @abstract
			 * Initialize the <code>MacroCommand</code>.
			 *
			 * <P>
			 * In your subclass, override this method to initialize the <code>MacroCommand</code>'s <i>subCommands</i>
			 * list with <code>Command</code> class references like this:
			 *
			 * <pre>
			 *    // Initialize MyMacroCommand
			 *    initializeMacroCommand: function()
			 *    {
			 *    	this.addSubCommand(FirstCommand);
			 *      this.addSubCommand(SecondCommand);
			 *      this.addSubCommand(ThirdCommand);
			 *    }
			 * </pre>
			 *
			 * <P>
			 * Note that <i>subCommands</i> may be any <code>Command</code> implementor.
			 *
			 * <P>
			 * In the JavaScript version it means that it only needs to implement an execute method and inherits from
			 * <code>Notifier</code>.
			 */
			initializeMacroCommand: function(){},

			/**
			 * Add an entry to <i>subCommands</i> list.
			 *
			 * <P>
			 * The <i>subCommands</i> will be called in First In/First Out (FIFO) order.
			 *
			 * @param {String} classPath
			 * 		Classpath of the <code>Command</code> class to add.
			 */
			addSubCommand: function( classPath )
			{
				this.subCommands.push( classPath );
			},

			/**
			 * Execute this <code>MacroCommand</code>'s <i>SubCommands</i>.
			 *
			 * <P>
			 * The <i>subCommands</i> will be called in First In/First Out (FIFO) order.
			 *
			 * @param {Notification} notification
			 * 		The <code>Notification</code> object to be passed to each entry	of <i>subCommands</i> list.
			 */
			execute: function( notification )
			{
				var len/*Number*/ = this.subCommands.length;
				for( var i/*Number*/=0; i<len; i++ )
				{
					var classPath/*String*/ = this.subCommands[i];
					var commandClass/*Function*/ = Objs(classPath);
					if( !commandClass )
						continue;

					var commandInstance/*Command*/ = new commandClass();
					commandInstance.setModel( this.model );
					commandInstance.setView( this.view );
					commandInstance.execute( notification );
				}
			}
		}
	);
}