/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Model = Objs("truemvc.mvc.Model");
	var View = Objs("truemvc.mvc.View");
	var Controller = Objs("truemvc.mvc.Controller");
	var Notification = Objs("truemvc.observer.Notification");
	
	/**
	 * @class Facade
	 *
	 * @classDescription
	 * The base <code>Facade</code> implementation for each core.
	 * 
	 * <P>
	 * In TrueMVC, the <code>Facade</code> class assumes these responsibilities:
	 * 
	 * <UL>
	 * 		<LI>Initializing the <code>Model</code>, <code>View</code> and <code>Controller</code> core actors.
	 * 		<LI>Providing all the applicable methods of the <code>Model</code>, <code>View</code>, &
	 * 		<code>Controller</code> instances.
	 * 		<LI>Providing a single point of contact to the application for registering <code>Command</code>s and
	 * 		notifying <code>Observer</code>s.
	 * 
	 * @see truemvc.mvc.Controller
	 * @see truemvc.mvc.Model
	 * @see truemvc.mvc.View
	 * @see truemvc.observer.Notification
	 * @see truemvc.mediator.Mediator
	 * @see truemvc.proxy.Proxy
	 * @see truemvc.command.Command
	 * @see truemvc.command.MacroCommand
	 *
	 * @constructor
	 */
	var Facade = Objs
	(
		"truemvc.facade.Facade",
		{
			/**
			 * The <code>View</code> for this <code>Facade</code>.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,
			
			/**
			 * The <code>Model</code> for this <code>Facade</code>.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,
			
			/**
			 * The <code>Controller</code> for this <code>Facade</code>.
			 *
			 * @type {Controller}
			 * @private
			 */
			controller: null,
			
			/**
			 * @constructs
			 * Initialize a <code>Facade</code> instance.
			 *
			 * Don't call it in your subclass for a custom initialization of the <code>Facade</code> that does not
			 * require the <code>Facade</code> to be automatically initialized with <code>initializeFacade</code>.
			 */
			initialize: function()
			{
				this.initializeFacade();
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Facade</code>.
			 *
			 * @return {Model}
			 * 		The <code>View</code> instance set for this <code>Facade</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Facade</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Facade</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * Get the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Command</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Command</code>.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Command</code>.
			 */
			setView: function( view )
			{
				this.view = view;
			},

			/**
			 * Get the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @return {Controller}
			 * 		The <code>Controller</code> instance set for this <code>Command</code>.
			 */
			getController: function()
			{
				return this.controller;
			},

			/**
			 * Set the <code>Controller</code> instance for this <code>Command</code>.
			 *
			 * @param {Controller} controller
			 * 		The <code>Model</code> instance to set for this <code>Command</code>.
			 */
			setController: function( controller )
			{
				this.controller = controller;
			},

			/**
			 * Automatically initialize the <code>Facade</code> instance with its <code>Model</code>, <code>View</code>,
			 * <code>Controller</code>.
			 *
			 * <P>
			 * Called automatically by the constructor.
			 */
			initializeFacade: function()
			{
				var model/*Model*/ = new Model();
				var view/*View*/ = new View();
				var controller/*Controller*/ = new Controller();

				this.setModel( model );
				this.setView( view );
				this.setController( controller );

				this.initializeModel();
				this.initializeView();
				this.initializeController();
			},

			/**
			 * Initialize the <code>Model</code>.
			 *
			 * <P>
			 * Called by the <code>initializeFacade</code> method.
			 *
			 * Override this method in your subclass of <code>Facade</code>  if one or both of the following are true:
			 * <UL>
			 * 		<LI> You wish to initialize a different <code>Model</code>.
			 * 		<LI> You have <code>Proxy</code>s to register with the Model that do not retrieve a reference to the
			 * 		Facade at construction time.
			 *
			 * If you don't want to initialize a different <code>Model</code>, call
			 * <code>$super.initializeModel()</code> at the beginning of your method, then register <code>Proxy</code>s.
			 *
			 * <P>
			 * Note: This method is <i>rarely</i> overridden; in practice you are more likely to use a
			 * <code>Command</code> to create and register <code>Proxy</code>s with the <code>Model</code>, since
			 * <code>Proxy</code>s with mutable data will likely need to send <code>Notification</code>s and thus will
			 * likely want to fetch a reference to the <code>Facade</code> during their construction.
			 */
			initializeModel: function()
			{
				//FIXME any previously registered model has to be cleanly removed (dedicated method)

				//In case the Model is set after the Controller
				if( this.controller )
					this.controller.setModel(this.model);

				//In case the Model is set after the View
				if( this.view )
					this.view.setModel(this.model);
			},

			/**
			 * Initialize the <code>View</code>.
			 *
			 * <P>
			 * Called by the <code>initializeFacade</code> method.
			 *
			 * Override this method in your subclass of <code>Facade</code> if one or both of the following are true:
			 * <UL>
			 * 		<LI> You wish to initialize a different <code>View</code>.
			 * 		<LI> You have <code>Observers</code> to register with the <code>View</code>
			 *
			 * If you don't want to initialize a different <code>View</code>, call <code>super.initializeView()</code>
			 * at the beginning of your method, then register <code>Mediator</code> instances.
			 *
			 * <P>
			 * Note: This method is <i>rarely</i> overridden; in practice you are more likely to use a
			 * <code>Command</code> to create and register <code>Mediator</code>s with the <code>View</code>, since
			 * <code>Mediator</code> instances will need to send <code>Notification</code>s and thus will likely want
			 * to fetch a reference to the <code>Facade</code> during their construction.
			 */
			initializeView: function()
			{
				//FIXME any previously registered view has to be cleanly removed (dedicated method)

				//In case the View is set after the Model
				if( this.model )
					this.view.setModel(this.model);

				//In case the View is set after the Controller
				if( this.controller )
					this.controller.setView(this.view);
			},

			/**
			 * Initialize the <code>Controller</code>.
			 *
			 * <P>
			 * Called by the <code>initializeFacade</code> method.
			 *
			 * Override this method in your subclass of <code>Facade</code> if one or both of the following are true:
			 * <UL>
			 * 		<LI> You wish to initialize a different <code>Controller</code>.
			 * 		<LI> You have <code>Commands</code> to register with the <code>Controller</code> at startup.</code>.
			 *
			 * If you don't want to initialize a different <code>Controller</code>, call
			 * <code>$super.initializeController()</code> at the beginning of your method, then register
			 * <code>Command</code>s.
			 */
			initializeController: function()
			{
				//FIXME any previously registered controller has to be cleanly removed (dedicated method)

				//In case the Controller is set after the View
				if( this.view )
					this.controller.setView(this.view);

				//In case the Controller is set after the Model
				if( this.model )
					this.controller.setModel(this.model);
			},

			/**
			 * Register a <code>Command</code> with the <code>Controller</code> by <code>Notification</code> name.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code> to associate the <code>Command</code> with.
			 *
			 * @param {String} classPath
			 * 		The class path of the <code>Command</code> to register.
			 */
			registerCommand: function( name, classPath )
			{
				this.controller.registerCommand( name, classPath );
			},
			
			/**
			 * Remove a previously registered <code>Command</code> to <code>Notification</code> mapping from the
			 * <code>Controller</code>.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code> to remove the <code>Command</code> mapping for.
			 */
			removeCommand: function( name )
			{
				this.controller.removeCommand(name);
			},

			/**
			 * Remove all previously registered <code>Command</code>s or <code>MacroCommand</code>s to
			 * <code>Notification</code> mapping in a one time pass.
			 *
			 * <P>
			 * Rem: Call this to mark the Controller as available for garbage collection, as it will remove itself
			 * for being referenced in Observers object detained by the view.
			 */
			removeAllCommands: function()
			{
				this.controller.removeAllCommands();
			},
			
			/**
			 * Check if a <code>Command</code> is registered for a given <code>Notification</code>.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code> to verify for the existence of a <code>Command</code>
			 * 		mapping for.
			 *
			 * @return {Boolean}
			 * 		A <code>Command</code> is currently registered for the given <i>name</i>.
			 */
			hasCommand: function( name )
			{
				return this.controller.hasCommand(name);
			},
			
			/**
			 * Register a <code>Proxy</code> with the <code>Model</code> by name.
			 *
			 * @param proxy {Proxy}
			 * 		The <code>Proxy</code> instance to be registered with the <code>Model</code>.
			 */
			registerProxy: function( proxy )
			{
				this.model.registerProxy( proxy );
			},
			
			/**
			 * Retrieve a <code>Proxy</code> from the <code>Model</code> by name.
			 *
			 * @param {String} proxyName
			 * 		The name of the <code>Proxy</code> to be retrieved.
			 *
			 * @return {Proxy}
			 * 		The <code>Proxy</code> instance previously registered with the given <i>proxyName</i>.
			 */
			retrieveProxy: function( proxyName )
			{
				return this.model.retrieveProxy(proxyName);
			},
			
			/**
			 * Remove an <code>Proxy</code> from the <code>Model</code> by name.
			 *
			 * @param {String} proxyName
			 * 		The <code>Proxy</code> to remove from the <code>Model</code>.
			 *
			 * @return {Proxy}
			 * 		The <code>Proxy</code> that was removed from the <code>Model</code>.
			 */
			removeProxy: function( proxyName )
			{
				return this.model.removeProxy(proxyName);
			},

			/**
			 * Remove each registered <code>Proxy</code> from the <code>Model</code>.
			 *
			 * <P>
			 * Rem: Call this to mark the Model as available for garbage collection, as it will remove itself
			 * for being referenced in each <code>Proxy</code> previously registered with it.
			 */
			removeAllProxies: function()
			{
				this.model.removeAllProxies();
			},

			/**
			 * Check if a <code>Proxy</code> is registered.
			 *
			 * @param {String} proxyName
			 * 		The <code>Proxy</code> to verify the existence of a registration with the <code>Model</code>.
			 *
			 * @return {Boolean}
			 * 		A <code>Proxy</code> is currently registered with the given <i>proxyName</i>.
			 */
			hasProxy: function( proxyName )
			{
				return this.model.hasProxy(proxyName);
			},
			
			/**
			 * Register a <code>Mediator</code> with the <code>View</code>.
			 *
			 * @param {Mediator} mediator
			 * 		A reference to the <code>Mediator</code>.
			 */
			registerMediator: function( mediator )
			{
				this.view.registerMediator( mediator );
			},
			
			/**
			 * Retrieve an <code>Mediator</code> from the <code>View</code>.
			 *
			 * @param {String} mediatorName
			 * 		The name of the registered <code>Mediator</code> to retrieve.
			 *
			 * @return {Mediator}
			 * 		The <code>Mediator</code> previously registered with the given <i>mediatorName</i>.
			 */
			retrieveMediator: function( mediatorName )
			{
				return this.view.retrieveMediator(mediatorName);
			},
			
			/**
			 * Remove an <code>Mediator</code> from the <code>View</code>.
			 *
			 * @param {String} mediatorName
			 * 		The name of the <code>Mediator</code> to be removed.
			 *
			 * @return {Mediator}
			 * 		The <code>Mediator</code> that was removed from the <code>View</code>.
			 */
			removeMediator: function( mediatorName )
			{
				return this.view.removeMediator(mediatorName);
			},
			
			/**
			 * Check if a <code>Mediator</code> is registered or not.
			 *
			 * @param {String} mediatorName
			 * 		The name of the <code>Mediator</code> to verify the existence of a registration for.
			 *
			 * @return {Boolean}
			 * 		A <code>Mediator</code> is registered with the given <i>mediatorName</i>.
			 */
			hasMediator: function( mediatorName )
			{
				return this.view.hasMediator(mediatorName);
			}
		}
	);
}