/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Notification = Objs("truemvc.observer.Notification");

	/**
	 * @class truemvc.mediator.Mediator
	 * @classDescription
	 * <P>A base <code>Mediator</code> implementation.
	 * 
	 * <P>
	 * Typically, a <code>Mediator</code> will be written to serve one specific control or group controls and so, will
	 * not have a need to be dynamically named.
	 * 
	 * @see truemvc.observer.Notification
	 * @implements truemvc.interfaces.INotifier
	 * @constructor
	 */
	var Mediator = Objs
	(
		"truemvc.mediator.Mediator",
		"truemvc.observer.Observer",
		{
			/**
			 * The <code>View</code> this <code>Mediator</code> has to know.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,

			/**
			 * The <code>Model</code> this <code>Mediator</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * The name of the <code>Mediator</code>.
			 * 
			 * @type {String}
			 * @private
			 */
			mediatorName: null,
			
			/**
			 * The <code>Mediator</code>'s view component.
			 * 
			 * @type {Object}
			 * @private
			 */
			viewComponent: null,
			
			/**
			 * @constructs
			 * @override
			 *
			 * Initialize a <code>Mediator</code> instance.
			 *
			 * @param {String} mediatorName
			 * 		The name of the <code>Mediator</code>.
			 *
			 * @param {Object} viewComponent
			 * 		The <code>Mediator</code>'s view component.
			 *
			 */
			initialize: function( mediatorName, viewComponent )
			{
				this.mediatorName = (mediatorName != null) ? mediatorName : Mediator.NAME;
				this.viewComponent = viewComponent;
			},

			/**
			 * Get the <code>View</code> instance for this <code>Mediator</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Mediator</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Mediator</code>.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Mediator</code>.
			 */
			setView: function( view )
			{
				this.view = view;
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Mediator</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Mediator</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Mediator</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Mediator</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * @override
			 */
			sendNotification: function( name, body, type )
			{
				var notification/*Notification*/ = new Notification( name, body, type );
				this.view.notifyObservers(notification);
			},

			/**
			 * List the <code>Notification</code> names this <code>Mediator</code> is interested in being notified of.
			 *
			 * @return {Array}
			 * 		The list of notifications names in which is interested the
			 * 		<code>Mediator</code>.
			 */
			listNotificationInterests: function()
			{
				return [];
			},
			
			/**
			 * Get the name of the <code>Mediator</code>.
			 *
			 * @return {String}
			 * 		The <code>Mediator</code> name.
			 */
			getMediatorName: function()
			{
				return this.mediatorName;
			},
			
			/**
			 * Get the <code>Mediator</code>'s view component.
			 *
			 * @return {Object}
			 * 		The view component.
			 */
			getViewComponent: function()
			{
				return this.viewComponent;
			},
			
			/**
			 * Set the <code>Mediator</code>'s view component.
			 *
			 * @param {Object} viewComponent
			 * 		The view component.
			 */
			setViewComponent: function( viewComponent )
			{
				this.viewComponent = viewComponent;
			},

			/**
			 * @override
			 * Handle <code>Notification</code>s.
			 *
			 * <P>
			 * Typically this will be handled in a switch statement, with one 'case' entry per <code>Notification</code>
			 * the <code>Mediator</code> is interested in.
			 *
			 * @param {Notification} notification
			 * 		The notification instance to be handled.
			 */
			handleNotification: function( notification ){},
			
			/**
			 * Called by the View when the Mediator is registered.
			 *
			 * <P>
			 * This method is usually overridden as needed by the subclass.
			 */
			onRegister: function(){},
			
			/**
			 * Called by the View when the Mediator is removed.
			 *
			 * <P>
			 * This method is usually overridden as needed by the subclass.
			 */
			onRemove: function(){}
		}
	);
	
	/**
	 * Default name of the <code>Mediator</code>.
	 * 
	 * @type {String}
	 * @constant
	 */
	Mediator.NAME = "Mediator";
}