/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Observer = Objs("truemvc.observer.Observer");
	var View = Objs("truemvc.mvc.View");

	/**
	 * @class truemvc.mvc.Controller
	 *
	 * @classDescription
	 * The <code>Controller</code> class for TrueMVC.
	 *
	 * <P>
	 * In TrueMVC, the <code>Controller</code> class follows the 'Command and Controller' strategy, and assumes these
	 * responsibilities:
	 *
	 * <UL>
	 * <LI>Remembering which <code>Command</code>s or <code>MacroCommand</code>s are intended to handle which
	 * <code>Notification</code>s.
	 *
	 * <LI>Registering itself as an <code>Observer</code> with the <code>View</code> for each <code>Notification</code>
	 * that it has a <code>Command</code> or <code>MacroCommand</code> mapping for.
	 *
	 * <LI>Creating a new instance of the proper <code>Command</code> or <code>MacroCommand</code> to handle a
	 * given <code>Notification</code> when notified by the <code>View</code>.
	 *
	 * <LI>Calling the <code>Command</code>'s or <code>MacroCommand</code>'s  <code>execute</code> method, passing
	 * in the <code>Notification</code>.
	 *
	 * <P>
	 * Your application must register <code>Command</code>s with the <code>Controller</code>.
	 *
	 * <P>
	 * The simplest way is to subclass </code>Facade</code>, and use its <code>initializeController</code> method to add
	 * your registrations.
	 *
	 * @see truemvc.View
	 * @see truemvc.observer.Observer
	 * @see truemvc.observer.Observable
	 * @see truemvc.observer.Notification
	 * @see truemvc.command.Command
	 * @see truemvc.command.MacroCommand
	 *
	 * @constructor
	 */
	var Controller = Objs
	(
		"truemvc.mvc.Controller",
		"truemvc.observer.Observer",
		{
			/**
			 * The <code>View</code> this <code>Controller</code> has to know.
			 *
			 * @type {View}
			 * @private
			 */
			view: null,

			/**
			 * The <code>Model</code> this <code>Controller</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * Mapping of <code>Notification<code> names to <code>Command</code> class references.
			 *
			 * @type {Object}
			 * @private
			 */
			commandMap: null,

			/**
			 * @constructs
			 * Initialize a <code>Controller</code> instance.
			 */
			initialize: function()
			{
				this.commandMap = {};
				this.observer = new Observer( this, this.executeCommand );
			},

			/**
			 * Get the <code>View</code> instance for this <code>Controller</code>.
			 *
			 * @return {View}
			 * 		The <code>View</code> instance set for this <code>Controller</code>.
			 */
			getView: function()
			{
				return this.view;
			},

			/**
			 * Set the <code>View</code> instance for this <code>Controller</code>.
			 *
			 * <P>
			 * Setting it to <code>null</code> will not make the <code>View</code> instance available for garbage as it
			 * is passed as reference to commands. Be sure to unregister all commands to make it available for garbage.
			 *
			 * @param {View} view
			 * 		The <code>View</code> instance to set for this <code>Controller</code>.
			 */
			setView: function( view )
			{
				//TODO throws an error if view is re-registered ?
				this.view = view;
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Controller</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Controller</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Controller</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Controller</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * @override
			 * If a <code>Command</code> has previously been registered to handle the given <code>Notification</code>,
			 * then it is executed.
			 *
			 * @param {Notification} notification
			 * 		A <code>Notification</code>.
			 */
			handleNotification: function( notification )
			{
				var notificationName/*String*/ = notification.getName();

				if( this.hasCommand(notificationName) )
				{
					var classPath/*String*/ = this.commandMap[notificationName];

					//Objs will throw an error if the classpath is not associated with a class.
					//TODO Catch and throw a framework standard error when associated command doesn't exist.
					var commandClass/*Function*/ = Objs(classPath);

					//TODO Catch and throw a framework standard error when associated class is not a Command.
					var command/*Command*/ = new commandClass();

					command.setModel(this.model);
					command.setView(this.view);
					command.execute(notification);
				}
			},

			/**
			 * Register a particular <code>Command</code> class as the handler for a particular
			 * <code>Notification</code>.
			 *
			 * <P>
			 * If a <code>Command</code> has already been registered to handle <code>Notification</code>s with this
			 * name, it is no longer used, the new <code>Command</code> is used instead.
			 *
			 * The <code>Observer</code> for the new <code>Command</code> is only created if this is the first time a
			 * <code>Command</code> has been registered for this <code>Notification</code> name.
			 *
			 * @param {String} notificationName
			 * 		The name of the <code>Notification</code>.
			 *
			 * @param {String} classPath
			 * 		The classpath of the <code>Command</code> class to register.
			 */
			registerCommand: function( notificationName, classPath )
			{
				//TODO If we authorize transparent re-registration here, we have to cleanly remove any previously registered command (and mainly reference its observers)
				if( this.hasCommand(notificationName) )
					return; //TODO Possibly throws an error or return a boolean if some languages doesn't support errors

				if( !this.view )
					return; //TODO Possibly throws an error or return a boolean if some languages doesn't support errors

				this.view.registerObserver( notificationName, this );
				this.commandMap[notificationName] = classPath;
			},

			/**
			 * Check if a <code>Command</code> is registered for a given <code>Notification</code>.
			 *
			 * <P>
			 * Rem: Do not verify if the class object associated to the command classpath exists.
			 *
			 * @param {String} notificationName
			 * 		The name of the <code>Notification</code> to verify the	existence of its registration.
			 *
			 * @return {Boolean}
			 * 		A <code>Command</code> is currently registered for the given <code>notificationName</code>.
			 */
			hasCommand: function( notificationName )
			{
				return typeof this.commandMap[notificationName] == "string";
			},

			/**
			 * Remove a previously registered <code>Command</code> or <code>MacroCommand</code> to
			 * <code>Notification</code> mapping.
			 *
			 * @param {String} notificationName
			 * 		The name of the <code>Notification</code> to remove the	<code>Command</code> or
			 * 		<code>MacroCommand</code> mapping for.
			 */
			removeCommand: function( notificationName )
			{
				if( this.hasCommand(notificationName) )
				{
					if( this.view )
						this.view.removeObserver( notificationName, this.observer );

					delete this.commandMap[notificationName];
				}
			},

			/**
			 * Remove all registered <code>Command</code>s or <code>MacroCommand</code>s to
			 * <code>Notification</code> mapping.
			 *
			 * <P>
			 * Rem: Call this to mark the Controller as available for garbage collection, as it will remove itself
			 * for being referenced in <code>Observer</code>s object detained by the view.
			 */
			removeAllCommands: function()
			{
				//Make a copy of the commandMap to be sure it will not be modified during the loop.
				var arr/*Array*/ = [];
				for( var notificationName/*String*/ in this.commandMap )
					arr.push(notificationName);

				var i/*Number*/ = arr.length;
				while( i-- )
					this.removeCommand(arr[i]);
			}
		}
	);
}