/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	var Observer = Objs("truemvc.observer.Observer");
	var Observable = Objs("truemvc.observer.Observable");
	var Proxy = Objs("truemvc.proxy.Proxy");

	/**
	 * @class truemvc.mvc.Model
	 *
	 * @classDescription
	 * The <code>Model</code> class for TrueMVC.
	 *
	 * <P>
	 * In TrueMVC, the <code>Model</code> class provides access to model objects (Proxies) by named lookup.
	 *
	 * <P>
	 * The <code>Model</code> assumes these responsibilities:
	 *
	 * <UL>
	 * 	<LI>Maintain a cache of <code>Proxy</code> instances.
	 * 	<LI>Provide methods for registering, retrieving, and removing <code>Proxy</code> instances.
	 *
	 * <P>Your application must register <code>Proxy</code> instances with the <code>Model</code>. Typically, you use a
	 * <code>Command</code> to create and register <code>Proxy</code> instances once the <code>Facade</code> has
	 * initialized the core actors.
	 *
	 * @see truemvc.proxy.Proxy
	 * @see truemvc.observer.Observable
	 *
	 * @constructor
	 */
	var Model = Objs
	(
		"truemvc.mvc.Model",
		{
			/**
			 * HashTable of <code>Proxy</code> instances registered with the <code>Model</code>.
			 *
			 * @type {Object}
			 * @private
			 */
			proxyMap: null,

			/**
			 * @private
			 *
			 * Observable composite object for the <code>View></code>.
			 *
			 * @type {Observable}
			 */
			observable: null,

			/**
			 * @constructs
			 * Initialize a <code>Model</code> instance.
			 */
			initialize: function()
			{
				this.proxyMap = {};
				this.observable = new Observable();
			},

			/**
			 * Register an <code>Observer</code> to be notified of <code>Notifications</code> with a given name.
			 *
			 * @param {String} name
			 * 		The name of the <code>Notification</code>s to notify this <code>Observer</code> of.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> to register.
			 */
			registerObserver: function( name, observer )
			{
				this.observable.registerObserver( name, observer );
			},

			/**
			 * Notify the <code>Observer</code>s for a particular notification (name).
			 *
			 * <P>
			 * All previously attached <code>Observer</code>s for this notification list are notified and
			 * are passed a reference to the <code>Notification</code> in the order in which they were registered.
			 *
			 * @param {Notification} notification
			 * 		The notification object to pass when notifying <code>Observer</code>s.
			 */
			notifyObservers: function( notification )
			{
				this.observable.notifyObservers( notification );
			},

			/**
			 * Remove any <code>Observer</code> registered for the given <code>Notification</code> name targeting the
			 * same <code>Observer</code> context, same <code>Observer</code> notify method.
			 *
			 * @param {String} name
			 * 		Notification name which will not be observed anymore with this <code>Observer</code>.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> which will not observe this notification anymore.
			 */

			removeObserver: function( name, observer )
			{
				this.observable.removeObserver( name, observer );
			},

			/**
			 * Remove all <code>Observer</code>s registered with this <code>Observable</code> instance.
			 *
			 * <P>
			 * Mainly used to mark current instance as candidate for Garbage Collection.
			 */
			removeAllObservers: function()
			{
				this.observable.removeAllObservers();
			},

			/**
			 * Search for any previously registered <code>Observer</code> with the same notification.
			 *
			 * @param {String} name
			 * 		The notification to search for registered <code>Observer</code>.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> to search for.
			 */
			hasObserver: function( name, observer )
			{
				return this.observable.hasObserver( name, observer );
			},

			/**
			 * Register a <code>Proxy</code> with the <code>Model</code>.
			 *
			 * @param {Proxy} proxy
			 *		A <code>Proxy</code> to be held by the <code>Model</code>.
			 */
			registerProxy: function( proxy )
			{
				var name/*String*/ = proxy.getProxyName();
				if( this.hasProxy(name) )
					return; //TODO Possibly throws an error or return a boolean if some languages doesn't support errors

 				this.proxyMap[name] = proxy;

				proxy.setModel(this);
				proxy.onRegister();
			},

			/**
			 * Retrieve an <code>Proxy</code> from the <code>Model</code>.
			 *
			 * @param {String} proxyName
			 *		The name of the <code>Proxy</code> to retrieve.
			 *
			 * @return {Proxy}
			 *		The <code>Proxy</code> instance previously registered with the given <code>proxyName</code> or an
			 *		explicit <code>null</code> if it doesn't exists.
			 */
			retrieveProxy: function( proxyName )
			{
				//Return a strict null when the proxy doesn't exist
				if( !this.hasProxy(proxyName) )
					return null;

				return this.proxyMap[proxyName];
			},

			/**
			 * Check if a <code>Proxy</code> is registered.
			 *
			 * @param {String} proxyName
			 *		The name of the <code>Proxy</code> to verify the existence of its registration.
			 *
			 * @return {Boolean}
			 *		A Proxy is currently registered with the given <code>proxyName</code>.
			 */
			hasProxy: function( proxyName )
			{
				return this.proxyMap[proxyName] instanceof Proxy;
			},

			/**
			 * Remove a <code>Proxy</code> from the <code>Model</code>.
			 *
			 * @param {String} proxyName
			 *		The name of the <code>Proxy</code> instance to be removed.
			 *
			 * @return {Proxy}
			 *		The <code>Proxy</code> that was removed from the <code>Model</code> or an explicit <code>null</null>
			 *		if the <code>Proxy</code> didn't exist.
			 */
			removeProxy: function( proxyName )
			{
				if( !this.hasProxy(proxyName) )
					return null;

				var proxy/*Proxy*/ = this.proxyMap[proxyName];
				delete this.proxyMap[proxyName];
				proxy.onRemove();

				//TODO Call instead kind of finalize method ?
				proxy.setModel(null);
				return proxy;
			},

			/**
			 * Remove each registered <code>Proxy</code>.
			 *
			 * <P>
			 * Rem: Call this to mark the Model as available for garbage collection, as it will remove itself
			 * for being referenced in each <code>Proxy</code> previously registered with it.
			 */
			removeAllProxies: function()
			{
				//Make a copy of the proxyMap to be sure it will not be modified during the loop.
				var arr/*Array*/ = [];
				for( var proxyName/*String*/ in this.proxyMap )
					arr.push(proxyName);

				var i/*Number*/ = arr.length;
				while( i-- )
					this.removeProxy(arr[i]);
			}
		}
	);
}