/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.observer.Observable
	 *
	 * @classDescription
	 * The base <code>Observable</code> class.
	 *
	 * <P>
	 * An <code>Observable</code> is an object that will broadcast information to interested objects. It offers public
	 * methods for interested objects to be able to subscribe to broadcasting and others to allow classes to compose
	 * with.
	 * 
	 * @constructor
	 *
	 * @implements truemvc.interfaces.IObservable
	 */
	Objs
	(
		"truemvc.observer.Observable",
		{
			/**
			 * @private
			 * List of <code>Observer</code> instances interested in <code>Notification</code>s.
			 *
			 * @type {Object}
			 */
			observerMap: null,

			//REM Languages (as ActionScript) which can use an object other as String as key for a hashmap can maitain an hashmap with observer as key to optimize operations when needed

			/**
			 * @constructs
			 * Initialize an <code>Observable</code> instance.
			 */
			initialize: function()
			{
				this.observerMap = {};
			},

			/**
			 * Register an <code>Observer</code> to be notified of <code>Notification</code>s with a given name.
			 *
			 * @param {String} name
			 * 		The name of the notification to notify this <code>Observer</code> of.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> to register.
			 */
			registerObserver: function( name, observer )
			{
				var observers/*Array*/ = this.observerMap[name];
				if( observers instanceof Array )
				{
					//We do not want the same observer to be registered more than one time for the same notification.
					if( !this.hasObserver( name, observer ) )
						observers.push(observer);
				}
				else
					this.observerMap[name] = [observer];
			},

			/**
			 * Notify the <code>Observer</code>s for a particular notification (name).
			 *
			 * <P>
			 * All previously attached <code>Observer</code>s for this notification list are notified and
			 * are passed a reference to the <code>Notification</code> in the order in which they were registered.
			 *
			 * @param {Notification} notification
			 * 		The notification object to pass when notifying <code>Observer</code>s.
			 */
			notifyObservers: function( notification )
			{
				var name/*String*/ = notification.getName();
				var observersRef/*Array*/ = this.observerMap[name];
				if( observersRef instanceof Array )
				{
					// Copy the array to avoid its modification during the loop.
					var observers/*Array*/ = observersRef.slice(0);
					var len/*Number*/ = observers.length;
					for( var i/*Number*/=0; i<len; i++ )
					{
						var observer/*Observer*/ = observers[i];
						observer.handleNotification(notification);
					}
				}
			},

			/**
			 * Remove any <code>Observer</code> registered for the given <code>Notification</code> name targeting the
			 * same <code>Observer</code> context, same <code>Observer</code> notify method.
			 *
			 * @param {String} name
			 * 		Notification name which will not be observed anymore with this <code>Observer</code>.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> which will not observe this notification anymore.
			 */
			removeObserver: function( name, observer )
			{
				var observers/*Array*/ = this.observerMap[name];
				if( !(observers instanceof Array) )
					return;

				var i/*Number*/ = observers.length;
				while( i-- )
				{
					if( observer === observers[i] )
					{
						//Let the Garbage Collector work for us (can probably be optimized for any specific langage).
						observers.splice( i, 1 );
						break;
					}
				}

				// Remove empty observer lists.
				if( observers.length == 0 )
					delete this.observerMap[name];
			},

			/**
			 * Search for any previously registered <code>Observer</code> with the same notification.
			 *
			 * @param {String} name
			 * 		The notification to search for registered <code>Observer</code>.
			 *
			 * @param {Observer} observer
			 * 		The <code>Observer</code> to search for.
			 */
			hasObserver: function( name, observer )
			{
				var observers/*Array*/ = this.observerMap[name];
				if( observers instanceof Array )
				{
					var i/*Number*/ = observers.length;
					while( i-- )
						if( observer === observers[i] )
							return true;
				}

				return false;
			},

			/**
			 * Remove all <code>Observer</code>s registered with this <code>Observable</code> instance.
			 *
			 * <P>
			 * Mainly used to mark current instance as candidate for Garbage Collection.
			 */
			removeAllObservers: function()
			{
				//Let the Garbage Collector work for us (can probably be optimized for any specific langage).
				this.observerMap = {};
			}
		}
	);
}