/*
 * TrueMVC JavaScript with Objs - www.tekool.net
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 */
new function()
{
	/**
	 * @class truemvc.proxy.Proxy
	 *
	 * @classDescription
	 * The base <code>Proxy</code> class.
	 * 
	 * <P>
	 * In TrueMVC, <code>Proxy</code> classes are used to manage parts of the application's data model.
	 *
	 * <P>
	 * A <code>Proxy</code> might simply manage a reference to a local data object, in which case interacting with it
	 * might involve setting and getting of its data in synchronous fashion.
	 *
	 * <P>
	 * <code>Proxy</code> classes are also used to encapsulate the application's interaction with remote services to
	 * store or retrieve data, in which case, we adopt an asynchronous idiom; setting data (or calling a method) on the
	 * <code>Proxy</code> and listening for a <code>Notification</code> to be sent when the <code>Proxy</code> has
	 * retrieved the data from the service.
	 *
	 * @see truemvc.mvc.Model
	 *
	 * @implements truemvc.interfaces.INotifier
	 * 
	 * @constructor
	 */
	var Proxy = Objs
	( 
		"truemvc.proxy.Proxy",
		{

			/**
			 * The <code>Model</code> this <code>Proxy</code> has to know.
			 *
			 * @type {Model}
			 * @private
			 */
			model: null,

			/**
			 * The data object controlled by the <code>Proxy</code>.
			 *
			 * @type {Object}
			 * @private
			 */
			data: null,
			
			/**
			 * The name of the <code>Proxy</code>.
			 * 
			 * @type {String}
			 * @private
			 */
			proxyName: null,

			/**
			 * @override
			 *
			 * Initialize a <code>Proxy</code> instance.
			 *
			 * @param {String} proxyName
			 * 		The name of the <code>Proxy</code>.
			 *
			 * @param {Object} data
			 * 		An initial data object to be held by the <code>Proxy</code>.
			 */
			initialize: function( proxyName, data )
			{
				this.proxyName = (proxyName != null) ? proxyName : Proxy.NAME;
				this.data = data;
			},

			/**
			 * Get the <code>Model</code> instance for this <code>Proxy</code>.
			 *
			 * @return {Model}
			 * 		The <code>Model</code> instance set for this <code>Proxy</code>.
			 */
			getModel: function()
			{
				return this.model;
			},

			/**
			 * Set the <code>Model</code> instance for this <code>Proxy</code>.
			 *
			 * @param {Model} model
			 * 		The <code>Model</code> instance to set for this <code>Proxy</code>.
			 */
			setModel: function( model )
			{
				this.model = model;
			},

			/**
			 * @override
			 */
			sendNotification: function( name, body, type )
			{
				var notification/*Notification*/ = new Notification( name, body, type );
				this.model.notifyObservers(notification);
			},

			/**
			 * Gets the proxyName.
			 *
			 * @return {String}
			 * 		The name of the proxy.
			 */
			getProxyName: function()
			{
				return this.proxyName;
			},

			/**
			 * Sets the data object.
			 *
			 * @param {Object} data
			 * 		The data to set.
			 */
			setData: function( data )
			{
				this.data = data;
			},

			/**
			 * Gets the data.
			 *
			 * @return {Object}
			 * 		The data held in the <code>Proxy.
			 */
			getData: function()
			{
				return this.data;
			},

			/**
			 * Called by the Model when the <code>Proxy</code> is registered.
			 *
			 * <P>
			 * This method is usually overridden as needed by the subclass.
			 */
			onRegister: function(){},

			/**
			 * Called by the Model when the <code>Proxy</code> is removed.
			 *
			 * <P>This method is usually overridden as needed by the subclass.
			 */
			onRemove: function(){}
		}
	);

	/**
	 * The default name of the <code>Proxy</code>
	 * 
	 * @type {String}
	 * @constant
	 */
	Proxy.NAME = "Proxy";
}