/*
 * Inspired from PureMVC - Copyright(c) 2006-2012 Futurescale, Inc., Some rights reserved.
 * TrueMVC JavaScript may be freely distributed under the MIT license.
 * For all details and documentation: https://github.com/tekool/truemvc-javascript
 */

/**
 * Test the TrueMVC Controller class.
 *
 * @see truemvc.mvc.ControllerTestVO ControllerTestVO
 * @see truemvc.mvc.ControllerTestCommand ControllerTestCommand
 */
var ControllerTest = new YUITest.TestCase
(
	{
	    /**
	     * The name of the test case - if not provided, one is automatically
	     * generated by the YUITest framework.
	     *
	     * @type {String}
	     * @private
	     */
	    name: "TrueMVC Controller class tests",

	    /**
	     * Sets up data that is needed by each test.
	     */
	    setUp: function()
		{
	    },

	    /**
	     * Cleans up everything that was created by setUp().
	     */
	    tearDown: function()
		{
	    },

		/**
		 * Tests Command registration and execution.
		 *
		 * <P>
		 * This test gets the Singleton Controller instance
		 * and registers the ControllerTestCommand class
		 * to handle 'ControllerTest' Notifications.<P>
		 *
		 * <P>
		 * It then constructs such a Notification and tells the
		 * Controller to execute the associated Command.
		 * Success is determined by evaluating a property
		 * on an object passed to the Command, which will
		 * be modified when the Command executes.
		 */
		testRegisterAndExecuteCommand: function()
		{
			var View = Objs("truemvc.mvc.View");
			var Controller = Objs("truemvc.mvc.Controller");
			var Notification = Objs("truemvc.observer.Notification");
			var ControllerTestVO = Objs("truemvc.mvc.ControllerTestVO");

			var view = new View();

			// Create the controller, register the ControllerTestCommand to handle 'ControllerTest' notifications
			var controller/*Controller*/ = new Controller();
			controller.setView(view);
			controller.registerCommand( 'ControllerTest', "truemvc.mvc.ControllerTestCommand" );

			// Create a 'ControllerTest' notification
			var vo/*Object*/ = new ControllerTestVO(12);
			var notification/*Notification*/ = new Notification( 'ControllerTest', vo );

			// Tell the controller to execute the Command associated with the notification
			// the ControllerTestCommand invoked will multiply the vo.input value
			// by 2 and set the result on vo.result
			controller.handleNotification(notification);

			// test assertions
			YUITest.Assert.areEqual
			(
				24,
				vo.result,
				"Expecting vo.result == 24"
			);
		},

		/**
		 * Tests Command registration and removal.
		 *
		 * <P>
		 * Tests that once a Command is registered and verified
		 * working, it can be removed from the Controller.
		 */
		testRegisterAndRemoveCommand: function()
		{
			var View = Objs("truemvc.mvc.View");
			var Controller = Objs("truemvc.mvc.Controller");
			var Notification = Objs("truemvc.observer.Notification");
			var ControllerTestVO = Objs("truemvc.mvc.ControllerTestVO");

			var view = new View();

			// Create the controller, register the ControllerTestCommand to handle 'ControllerTest' notifications
			var controller/*Controller*/ = new Controller();
			controller.setView(view);
			controller.registerCommand( 'ControllerRemoveTest', "truemvc.mvc.ControllerTestCommand" );

			// Create a 'ControllerTest' notification
			var vo/*Object*/ = new ControllerTestVO(12) ;
			var notification/*Notification*/ = new Notification( 'ControllerRemoveTest', vo );

			// Tell the controller to execute the Command associated with the note
			// the ControllerTestCommand invoked will multiply the vo.input value
			// by 2 and set the result on vo.result
			controller.handleNotification(notification);

			// test assertions
			YUITest.Assert.areEqual
			(
				24,
				vo.result,
				"Expecting vo.result == 24"
			);

			// Reset result
			vo.result = 0;

			// Remove the Command from the Controller
			controller.removeCommand('ControllerRemoveTest');

			// Tell the controller to execute the Command associated with the
			// notification. This time, it should not be registered, and our vo result
			// will not change
			controller.handleNotification(notification);

			// test assertions
			YUITest.Assert.areEqual
			(
				0,
				vo.result,
				"Expecting vo.result == 0"
			);
		},

		//TODO We need a dedicated test for removeAllCommands

		/**
		 * Test hasCommand method.
		 */
		testHasCommand: function()
		{

			var View = Objs("truemvc.mvc.View");
			var Controller = Objs("truemvc.mvc.Controller");
			var Notification = Objs("truemvc.observer.Notification");

			var view = new View();

			// register the ControllerTestCommand to handle 'hasCommandTest' notifications
			var controller/*Controller*/ = new Controller();
			controller.setView(view);
			controller.registerCommand( 'hasCommandTest', "truemvc.mvc.ControllerTestCommand" );

			// test that hasCommand returns true for hasCommandTest notifications
			YUITest.Assert.isTrue
			(
				controller.hasCommand('hasCommandTest'),
				"Expecting controller.hasCommand('hasCommandTest') === true"
			);

			// Remove the Command from the Controller
			controller.removeCommand('hasCommandTest');

			// test that hasCommand returns false for hasCommandTest notifications
			YUITest.Assert.isFalse
			(
				controller.hasCommand('hasCommandTest'),
				"Expecting controller.hasCommand('hasCommandTest') === false"
			);
		},

		/**
		 * Tests Removing and Reregistering a Command
		 *
		 * <P>
		 * Tests that when a Command is re-registered that it isn't fired twice.
		 * This involves, minimally, registration with the controller but
		 * notification via the View, rather than direct execution of
		 * the Controller's executeCommand method as is done above in
		 * testRegisterAndRemove. The bug under test was fixed in AS3 Standard
		 * Version 2.0.2. If you run the unit tests with 2.0.1 this
		 * test will fail.
		 */
		testReregisterAndExecuteCommand: function()
		{
			var View = Objs("truemvc.mvc.View");
			var Controller = Objs("truemvc.mvc.Controller");
			var Notification = Objs("truemvc.observer.Notification");

			var view = new View();

			// Fetch the controller, register the ControllerTestCommand2 to handle 'ControllerTest2' notifications
			var controller/*Controller*/ = new Controller();
			controller.setView(view);
			controller.registerCommand( 'ControllerTest2', "truemvc.mvc.ControllerTestCommand2" );

			// Remove the Command from the Controller
			controller.removeCommand('ControllerTest2');

			// Re-register the Command with the Controller
			controller.registerCommand( 'ControllerTest2', "truemvc.mvc.ControllerTestCommand2" );

			// Create a 'ControllerTest2' notification
			var vo/*Object*/ = new ControllerTestVO( 12 );
			var notification/*Notification*/ = new Notification( 'ControllerTest2', vo );

			// send the Notification
			view.notifyObservers(notification);

			// test assertions
			// if the command is executed once the value will be 24
			YUITest.Assert.areEqual
			(
				24,
				vo.result,
				"Expecting vo.result == 24"
			);

			// Prove that accumulation works in the VO by sending the notification again
			view.notifyObservers(notification);

			// if the command is executed twice the value will be 48
			YUITest.Assert.areEqual
			(
				48,
				vo.result,
				"Expecting vo.result == 48"
			);
		}
  	}
);